@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -93,6 +93,7 @@ void slrn_chmap_fix_headers (void)
 	  {
 	     chmap_map_string_from_iso (h->subject);
 	     chmap_map_string_from_iso (h->from);
+	     chmap_map_string_from_iso (h->realname);
 	     h->flags |= HEADER_CHMAP_PROCESSED;
 	  }
         h = h->real_next;
