@@ -15,14 +15,14 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLRN_CHMAP_H
 #define _SLRN_CHMAP_H
 extern int slrn_set_charset (char *);
 extern int slrn_chmap_fix_file (char *, int);
-extern void slrn_chmap_fix_body (Slrn_Article_Type *);
+extern void slrn_chmap_fix_body (Slrn_Article_Type *, int);
 extern void slrn_chmap_fix_headers (void);
 
 #if SLRN_HAS_CHARACTER_MAP
