@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -47,16 +47,17 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
    char ch = 'e';
    int ich;
    char file[256];
-   char qregexp[512];
+   char qregexp[512], buf[512];
    unsigned int mm = 0, dd = 0, yy = 0;
    int days = 0;
-   int use_expired = 0;
+   int use_expired = 0, force_score = 0;
    unsigned int linenum = 0;
    char *q, *ng = newsgroup;
    int score;
    FILE *fp;
    time_t myclock;
-   int file_modified = 0;
+   int file_modified = 0, re_error = 0;
+   SLRegexp_Type re;
 
    if (Slrn_Score_File == NULL)
      {
@@ -71,10 +72,15 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	ch |= 0x20;
 	if (ch == 'c') return -1;
    
-	score = -9999;
-	if (-1 == slrn_read_integer ("Score", &score, &score))
-	  return -1;
-	if (score == 0) return -1;
+	while (1)
+	  {
+	     *qregexp = 0;
+	     if (-1 == slrn_read_input ("Score: ", "=-9999", qregexp, 1, 0))
+	       return -1;
+	     force_score = (*qregexp == '=');
+	     if (1 == sscanf (qregexp + force_score, "%d", &score))
+	       break;
+	  }
 	
 	if ('a' == (0x20 | slrn_get_response ("TtaA", "Which newsgroups: \001This group, \001All groups")))
 	  ng = "*";
@@ -82,7 +88,7 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	while (1)
 	  {
 	     *qregexp = 0;
-	     if (-1 == slrn_read_input ("Expires (MM/DD/YYYY, DD-MM-YYYY, +NN (days) or leave blank)", NULL, qregexp, 1, 0))
+	     if (-1 == slrn_read_input ("Expires (MM/DD/YYYY, DD-MM-YYYY, +NN (days) or leave blank): ", NULL, qregexp, 1, 0))
 	       return -1;
 	     if (*qregexp)
 	       {
@@ -128,9 +134,15 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	return -1;
      }
    
+   re.pat = qregexp;
+   re.buf = buf;
+   re.buf_len = sizeof (buf);
+   re.case_sensitive = 0;
+   
    if (Slrn_Prefix_Arg_Ptr == NULL)
      {
 	char *subject;
+	int comment;
 	linenum = 1;
 	while (EOF != (ich = getc (fp)))
 	  {
@@ -138,8 +150,8 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	  }
 	
 	myclock = time((time_t *) 0);
-	fprintf(fp, "\n%%BOS\n%%Score created by slrn on %s\n[%s]\nScore: %d\n",
-		 (char *) ctime(&myclock), ng, score);
+	fprintf(fp, "\n%%BOS\n%%Score created by slrn on %s\n[%s]\nScore: %s%d\n",
+		(char *) ctime(&myclock), ng, force_score ? "=" : "", score);
 	
 	if (use_expired)
 	  fprintf (fp, "Expires: %u/%u/%u\n", mm, dd, yy);
@@ -147,33 +159,52 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 	
 	subject = slrn_safe_strmalloc (h->subject);
 	slrn_subject_strip_was (subject);
-	q = SLregexp_quote_string (subject, qregexp, sizeof (qregexp));
-	if (q == NULL) q = subject;
+	if ((NULL == (q = SLregexp_quote_string (subject, qregexp, sizeof (qregexp)))) ||
+	    (ch != 's') ||
+	    (0 != SLang_regexp_compile (&re)))
+	  {
+	     re_error |= (ch == 's');
+	     comment = 1;
+	  }
+	else
+	  comment = 0;
 	fprintf (fp, "%c\tSubject: %s\n",
-		 ((ch == 's') ? ' ' : '%'), q);
+		 (comment ? '%' : ' '), (q ? q : subject));
 	slrn_free (subject);
 	
-	q = SLregexp_quote_string (h->from, qregexp, sizeof (qregexp));
-	if (q == NULL) q = h->from;
-	fprintf (fp, "%c\tFrom: %s\n",
-		 ((ch == 'f') ? ' ' : '%'), q);
-	
-	q = SLregexp_quote_string (h->msgid, qregexp, sizeof (qregexp));
-	if (q == NULL) q = h->msgid;
-	fprintf (fp, "%c\tReferences: %s\n",
-		 ((ch == 'r') ? ' ' : '%'), q);
-	
-	
-	q = SLregexp_quote_string (h->xref, qregexp, sizeof(qregexp));
-	if (q != NULL)
+	if ((NULL == (q = SLregexp_quote_string (h->from, qregexp, sizeof (qregexp)))) ||
+	    (ch != 'f') ||
+	    (0 != SLang_regexp_compile (&re)))
 	  {
-	     /* The way slrn handles Xref, we only need this: */
-	     fprintf (fp, "%%\t%s\n", q);
+	     re_error |= (ch == 'f');
+	     comment = 1;
+	  }
+	else
+	  comment = 0;
+	fprintf (fp, "%c\tFrom: %s\n",
+		 (comment ? '%' : ' '), (q ? q : h->from));
+	
+	if ((NULL == (q = SLregexp_quote_string (h->msgid, qregexp, sizeof (qregexp)))) ||
+	    (ch != 'r') ||
+	    (0 != SLang_regexp_compile (&re)))
+	  {
+	     re_error |= (ch == 'r');
+	     comment = 1;
+	  }
+	else
+	  comment = 0;
+	fprintf (fp, "%c\tReferences: %s\n",
+		 (comment ? '%' : ' '), (q ? q : h->msgid));
+	
+	if (NULL != (q = SLregexp_quote_string (h->xref, qregexp, sizeof (qregexp))))
+	  {
+	     fprintf (fp, "%%\tXref: %s\n", q);
 	  }
 	
-	q = SLregexp_quote_string (newsgroup, qregexp, sizeof(qregexp));
-	if (q == NULL) q = newsgroup;
-	fprintf (fp, "%%\tNewsgroup: %s\n", q);
+	if (NULL != (q = SLregexp_quote_string (newsgroup, qregexp, sizeof (qregexp))))
+	  {
+	     fprintf (fp, "%%\tNewsgroup: %s\n", q);
+	  }
 	
 	fprintf (fp, "%%EOS\n");
 	file_modified = 1;
@@ -183,6 +214,12 @@ int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
 
    if (-1 == slrn_fclose (fp))
      return -1;
+   
+   if (re_error)
+     {
+	slrn_error ("Error in pattern - commented new scorefile entry out.");
+	return -1;
+     }
 
    if (ch == 'e')
      {
