@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -53,11 +53,11 @@
 #include "art.h"
 #include "misc.h"
 #include "post.h"
+#include "util.h"
 #include "server.h"
 #include "hash.h"
 #include "score.h"
 #include "menu.h"
-#include "util.h"
 #include "startup.h"
 #include "slrndir.h"
 #include "vfile.h"
@@ -293,7 +293,6 @@ static int group_update_range (Slrn_Group_Type *g, int min, int max) /*{{{*/
 	  group_mark_article_as_read (g, n);
 	
 	/* g->unread = 0; */
-	slrn_message ("No articles to read.");
 	Slrn_Full_Screen_Update = 1;
 	Slrn_Groups_Dirty = 1;
 	return -1;
@@ -599,7 +598,8 @@ static void remove_group_entry (Slrn_Group_Type *g) /*{{{*/
 }
 /*}}}*/
 static int add_group (char *name, unsigned int len, /*{{{*/
-		      unsigned int subscribe_flag, int create_flag)
+		      unsigned int subscribe_flag, int query_server,
+		      int create_flag)
 {
    char ch;
    Slrn_Group_Type *g;
@@ -618,7 +618,7 @@ static int add_group (char *name, unsigned int len, /*{{{*/
 	       }
 	     return -1;
 	  }
-	else g = create_group_entry (name, len, -1, -1,
+	else g = create_group_entry (name, len, -1, -1, query_server &&
 				     !(subscribe_flag & GROUP_UNSUBSCRIBED),
 				     0);
 	if (g == NULL) return -1;
@@ -741,7 +741,7 @@ static void insert_new_groups (void) /*{{{*/
 	  {
 	     ugnext = ug->next;
 	     
-	     if (-1 != add_group (ug->group_name, strlen (ug->group_name), subscribe_flag, 0))
+	     if (-1 != add_group (ug->group_name, strlen (ug->group_name), subscribe_flag, 0, 0))
 	       last_group = place_group_in_newsrc_order (last_group);
 	     
 	     free_unsubscribed_group_type (ug);
@@ -836,7 +836,7 @@ static void toggle_group_formats (void)
 					     Group_Format_Number);
 }
 
-int slrn_group_search (char *str) /*{{{*/
+int slrn_group_search (char *str, int dir) /*{{{*/
 {
    SLsearch_Type st;
    Slrn_Group_Type *g;
@@ -849,10 +849,16 @@ int slrn_group_search (char *str) /*{{{*/
    
    do
      {
-	g = g->next;
+	if (dir > 0)
+	  g = g->next;
+	else
+	  g = g->prev;
 	if (g == NULL)
 	  {
 	     g = Groups;
+	     if (dir < 0)
+	       while (g->next != NULL)
+		 g = g->next;
 	  }
 	
 	if ((g->flags & GROUP_HIDDEN) == 0)
@@ -914,7 +920,10 @@ int slrn_group_select_group (void) /*{{{*/
    last_n = Slrn_Group_Current_Group->unread;
 
    if (-1 == group_sync_group_with_server (Slrn_Group_Current_Group, &min, &max))
-     return -1;
+     {
+	slrn_message ("No articles to read.");
+	return -1;
+     }
    
    n = Slrn_Group_Current_Group->unread;
 
@@ -934,7 +943,7 @@ int slrn_group_select_group (void) /*{{{*/
      {
 	char int_prompt_buf[256];
 	slrn_snprintf (int_prompt_buf, sizeof (int_prompt_buf),
-		       "%s: Read how many?",
+		       "%s: Read how many? ",
 		       Slrn_Group_Current_Group->group_name);
 	if ((-1 == slrn_read_integer (int_prompt_buf, &n, &n))
 	    || (n <= 0))
@@ -1027,7 +1036,7 @@ void slrn_group_quit (void) /*{{{*/
 	  slrn_quit (1);
 
 	slrn_smg_refresh ();
-	if (Slrn_Batch == 0) sleep (2);
+	if (Slrn_Batch == 0) slrn_sleep (2);
 	if (slrn_get_yesno (0, "Write to newsrc file failed.  Quit anyway") <= 0)
 	  return;
      }
@@ -1091,14 +1100,75 @@ static void set_current_group (void) /*{{{*/
 
 /*}}}*/
 
+#define MAX_QUEUED_GROUPS 20
+static void refresh_groups (Slrn_Group_Type **c) /*{{{*/
+{
+   Slrn_Group_Type *g = Groups;
+   Slrn_Group_Range_Type ranges[MAX_QUEUED_GROUPS];
+   
+   while (g != NULL)
+     {
+	Slrn_Group_Type *start = g;
+	int i = 0, j = 0;
+	
+	while ((g != NULL) && (i < MAX_QUEUED_GROUPS))
+	  {
+	     if (!(g->flags & GROUP_UNSUBSCRIBED))
+	       {
+		  ranges[i].name = g->group_name;
+		  i++;
+	       }
+	     g = g->next;
+	  }
+	if (Slrn_Server_Obj->sv_refresh_groups (ranges, i))
+	  {
+	     slrn_error ("Server connection dropped.");
+	     return;
+	  }
+	g = start;
+	while (j < i)
+	  {
+	     if (g->flags & GROUP_UNSUBSCRIBED)
+	       {
+		  g = g->next;
+		  continue;
+	       }
+	     if (ranges[j].min == -1)
+	       {
+		  slrn_error ("Group %s is bogus%s.", g->group_name,
+			      Slrn_Drop_Bogus_Groups ? " - dropping it" : "");
+		  if (Slrn_Drop_Bogus_Groups)
+		    {
+		       Slrn_Group_Type *tmp = g->next;
+		       remove_group_entry (g);
+		       if (g == *c) *c = Groups;
+		       g = tmp;
+		       j++;
+		       continue;
+		    }
+	       }
+	     else
+	       group_update_range (g, ranges[j].min, ranges[j].max);
+	     g = g->next;
+	     j++;
+	  }
+	while ((g != NULL) && (g->flags & GROUP_UNSUBSCRIBED))
+	  g = g->next;
+     }
+}
+/*}}}*/
+
 static void refresh_groups_cmd (void) /*{{{*/
 {
    Slrn_Group_Type *c = Slrn_Group_Current_Group;
    Slrn_Group_Current_Group = NULL;
    
+   slrn_message_now ("Checking news%s ...",
+		     Slrn_List_Active_File ? " via active file" : "");
+   
    if (Slrn_List_Active_File)
      {
-	read_and_parse_active(0);
+	read_and_parse_active (0);
 	if (c != NULL)
 	  /* hack: avoid a problem with inn not updating the high water mark */
 	  {
@@ -1124,15 +1194,7 @@ static void refresh_groups_cmd (void) /*{{{*/
    
    if (!Slrn_List_Active_File)
      {
-	Slrn_Group_Type *g = Groups;
-	int min, max;
-	
-	while (g != NULL)
-	  {
-	     if (!(g->flags & GROUP_UNSUBSCRIBED))
-	       group_sync_group_with_server (g, &min, &max);
-	     g = g->next;
-	  }
+	refresh_groups (&c);
      }
 
    slrn_read_group_descriptions ();
@@ -1144,44 +1206,75 @@ static void refresh_groups_cmd (void) /*{{{*/
 
 /*}}}*/
 
-static void group_search_cmd (void) /*{{{*/
+static void generic_group_search (int dir) /*{{{*/
 {
    static char search_str[256];
+   char prompt[20];
    Slrn_Group_Type *g;
    unsigned int n;
    
    g = Slrn_Group_Current_Group;
    if (g == NULL) return;
    
-   if (slrn_read_input ("Search", search_str, NULL, 1, 0) <= 0) return;
+   sprintf (prompt, "%s Search: ", dir > 0 ? "Forward" : "Backward"); /* safe */
+   if (slrn_read_input (prompt, search_str, NULL, 1, 0) <= 0) return;
    
    n = Group_Window.line_num;
-   if (0 == slrn_group_search (search_str))
+   if (0 == slrn_group_search (search_str, dir))
      {
         slrn_error ("Not found.");
 	return;
      }
    
-   if (n > Group_Window.line_num)
+   if (((dir > 0) && (n > Group_Window.line_num)) ||
+       ((dir < 0) && (n < Group_Window.line_num)))
      slrn_message ("Search wrapped.");
 }
 
 /*}}}*/
 
+static void group_search_forward (void) /*{{{*/
+{
+   generic_group_search (1);
+}
+/*}}}*/
+
+static void group_search_backward (void) /*{{{*/
+{
+   generic_group_search (-1);
+}
+/*}}}*/
+
+int slrn_add_group (char *group) /*{{{*/
+{
+   int retval = 0;
+   if (!find_group (group))
+     {
+	if (Slrn_List_Active_File == 0)
+	  {
+	     retval = add_group (group, strlen (group), 0, 1, 0);
+	     slrn_read_group_descriptions ();
+	  }
+	else
+	  {
+	     slrn_error ("Group %s does not exist.", group);
+	     retval = -1;
+	  }
+     }
+   Slrn_Groups_Dirty = 1;
+   Slrn_Full_Screen_Update = 1;
+   find_line_num ();
+   return retval;
+}
+/*}}}*/
+
 static void add_group_cmd (void) /*{{{*/
 {
    char group[256];
    
    *group = 0;
-   if (slrn_read_input ("Add group", NULL, group, 1, 0) > 0)
-     {
-	if (!find_group (group)
-	    && (Slrn_List_Active_File == 0))
-	  add_group (group, strlen (group), 0, 0);
-	Slrn_Groups_Dirty = 1;
-	Slrn_Full_Screen_Update = 1;
-	find_line_num ();
-     }
+   if (slrn_read_input ("Add group: ", NULL, group, 1, 0) > 0)
+     (void) slrn_add_group (group);
 }
 
 /*}}}*/
@@ -1334,7 +1427,7 @@ static void subscribe (void) /*{{{*/
    
    Slrn_Prefix_Arg_Ptr = NULL;
    	
-   if (NULL == (re = read_group_regexp ("Subscribe pattern", NULL)))
+   if (NULL == (re = read_group_regexp ("Subscribe pattern: ", NULL)))
      return;
 
    g = Groups;
@@ -1405,7 +1498,7 @@ static void unsubscribe (void) /*{{{*/
    
    Slrn_Prefix_Arg_Ptr = NULL;
    	
-   if (NULL == (re = read_group_regexp ("Un-Subscribe pattern", NULL)))
+   if (NULL == (re = read_group_regexp ("Un-Subscribe pattern: ", NULL)))
      return;
 
    g = Groups;
@@ -1472,7 +1565,7 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 	SLRegexp_Type *re;
 	char origpat[256];
 	
-	if (NULL == (re = read_group_regexp ("List Groups (e.g., comp*unix*)",
+	if (NULL == (re = read_group_regexp ("List Groups (e.g., comp*unix*): ",
 					     origpat)))
 	  {
 	     all_hidden = 1;
@@ -1563,6 +1656,11 @@ static void toggle_list_all_groups (void) /*{{{*/
 
 /*}}}*/
 
+void slrn_list_all_groups (int mode)
+{
+   toggle_list_all_groups1 (!mode);
+}
+
 void slrn_hide_current_group (void) /*{{{*/
 {
    if (Slrn_Group_Current_Group == NULL) return;
@@ -1607,6 +1705,12 @@ static void toggle_hide_groups (void) /*{{{*/
 
 /*}}}*/
 
+void slrn_hide_groups (int mode)
+{
+   Groups_Hidden = !mode;
+   toggle_hide_groups ();
+}
+
 static void select_group_cmd (void)
 {
    if (-1 == slrn_group_select_group ())
@@ -1643,16 +1747,16 @@ void slrn_post_cmd (void) /*{{{*/
    if (strlen (name) >= sizeof (group))
      name = "";
    slrn_strncpy (group, name, sizeof (group));
-   if (slrn_read_input ("Newsgroup", NULL, group, 1, 1) <= 0) return;
+   if (slrn_read_input ("Newsgroup: ", NULL, group, 1, 1) <= 0) return;
    if (strchr (group, ',') != NULL)
      {
 	slrn_strncpy (followupto, name, sizeof (followupto));
-	(void) slrn_read_input ("Followup-To", NULL, followupto, 1, 1);
+	(void) slrn_read_input ("Followup-To: ", NULL, followupto, 1, 1);
      }
    else
      *followupto = '\0';
    
-   *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1, 0) <= 0) return;
+   *subj = 0; if (slrn_read_input ("Subject: ", NULL, subj, 1, 0) <= 0) return;
    
    (void) slrn_post (group, followupto, subj);
 }
@@ -1757,8 +1861,9 @@ static SLKeymap_Function_Type Group_Functions [] = /*{{{*/
    A_KEY("digit_arg", slrn_digit_arg),
    A_KEY("eob", group_eob),
    A_KEY("evaluate_cmd", slrn_evaluate_cmd),
-   A_KEY("group_search", group_search_cmd),
-   A_KEY("group_search_forward", group_search_cmd),
+   A_KEY("group_search", group_search_forward),
+   A_KEY("group_search_backward", group_search_backward),
+   A_KEY("group_search_forward", group_search_forward),
    A_KEY("help", slrn_group_help),
    A_KEY("line_down", group_down),
    A_KEY("line_up", group_up),
@@ -1968,7 +2073,8 @@ void slrn_init_group_mode (void) /*{{{*/
 #endif
    SLkm_define_key  ("N", (FVOID_STAR) group_down, Slrn_Group_Keymap);
    SLkm_define_key  ("^N", (FVOID_STAR) group_down, Slrn_Group_Keymap);
-   SLkm_define_key  ("/", (FVOID_STAR) group_search_cmd, Slrn_Group_Keymap);
+   SLkm_define_key  ("/", (FVOID_STAR) group_search_forward, Slrn_Group_Keymap);
+   SLkm_define_key  ("\\", (FVOID_STAR) group_search_backward, Slrn_Group_Keymap);
    SLkm_define_key  ("G", (FVOID_STAR) refresh_groups_cmd, Slrn_Group_Keymap);
    SLkm_define_key  ("X", (FVOID_STAR) save_newsrc_cmd, Slrn_Group_Keymap);
    
@@ -2036,8 +2142,8 @@ void slrn_get_group_descriptions (void) /*{{{*/
 #ifdef VMS
    slrn_snprintf (file, sizeof (file), "%s-dsc", Slrn_Newsrc_File);
 #else
-# if defined(IBMPC_SYSTEM)
-   slrn_snprintf (file, sizeof (file), "ds-%s", Slrn_Newsrc_File);
+# ifdef SLRN_USE_OS2_FAT
+   slrn_os2_make_fat (file, sizeof (file), Slrn_Newsrc_File, ".dsc");
 # else
    slrn_snprintf (file, sizeof (file), "%s.dsc", Slrn_Newsrc_File);
 # endif
@@ -2123,14 +2229,13 @@ int slrn_read_group_descriptions (void) /*{{{*/
 #ifdef VMS
    slrn_snprintf (file, sizeof (file), "%s-dsc", Slrn_Newsrc_File);
 #else
-# if defined(IBMPC_SYSTEM)
-   slrn_snprintf (file, sizeof (file), "ds-%s", Slrn_Newsrc_File);
+# ifdef SLRN_USE_OS2_FAT
+   slrn_os2_make_fat (file, sizeof (file), Slrn_Newsrc_File, ".dsc");
 # else
    slrn_snprintf (file, sizeof (file), "%s.dsc", Slrn_Newsrc_File);
 # endif
 #endif
    
-   
    if (NULL == (fp = fopen (file, "r")))
      {
 	if (Slrn_Lib_Dir != NULL)
@@ -2199,7 +2304,6 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
 # endif
 #endif
    
-   
    if ((create_flag == 0)
        && (NULL != (fp = fopen (file, "r"))))
      {
@@ -2257,36 +2361,24 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
 	*p = 0;
 	
 	parse_error = 0;
-	i = 0;
-	do
+	
+	switch (Slrn_Server_Obj->sv_put_server_cmd (line, line, sizeof (line)))
 	  {
-	     switch (Slrn_Server_Obj->sv_put_server_cmd (line, line, sizeof (line)))
-	       {
-		case OK_NEWGROUPS:
-		  break;
-		  
-		case ERR_FAULT:	     
-		  return -1;
-		  
-		case ERR_COMMAND:
-		  slrn_message ("Server does not implement NEWGROUPS command.");
-		  return 0;
-		  
-		case ERR_NOAUTH:
-		  if ((i++ != 0) || (-1 == Slrn_Server_Obj->sv_send_authinfo ()))
-		    {
-		       slrn_exit_error ("Authorization failed.");
-		       return -1;
-		    }
-		  break;
-		  
-		default:
-		  slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
-				line);
-		  goto parse_error_label;
-	       }
+	   case OK_NEWGROUPS:
+	     break;
+	     
+	   case ERR_FAULT:	     
+	     return -1;
+	     
+	   case ERR_COMMAND:
+	     slrn_message ("Server does not implement NEWGROUPS command.");
+	     return 0;
+	     
+	   default:
+	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
+			   line);
+	     goto parse_error_label;
 	  }
-	while (i++ == 1); /* loops (at most) once */
 	
 	num = 0;
 	
@@ -2439,7 +2531,7 @@ static void read_and_parse_active (int create_flag) /*{{{*/
 		  putc ('.', stdout);
 		  fflush (stdout);
 	       }
-	     add_group (line, len, GROUP_UNSUBSCRIBED, 1);
+	     add_group (line, len, GROUP_UNSUBSCRIBED, 0, 1);
 	  }
      }
 
@@ -2512,7 +2604,7 @@ static int read_and_parse_newsrc_file (void)
 	vline[vlen-1] = 0;	       /* kill \n and NULL terminate */
 
 	if (-1 == add_group (vline, (unsigned int) (p - vline),
-			     ((ch == '!') ? GROUP_UNSUBSCRIBED : 0), 0))
+			     ((ch == '!') ? GROUP_UNSUBSCRIBED : 0), 0, 0))
 	  continue;
 	     
 	/* perform a re-arrangement to match arrangement in the
@@ -2522,16 +2614,14 @@ static int read_and_parse_newsrc_file (void)
 	  last_group = place_group_in_newsrc_order (last_group);
      }
    vclose (vp);
+   refresh_groups (&Slrn_Group_Current_Group);
    return 0;
 }
    
 int slrn_read_newsrc (int create_flag) /*{{{*/
 {
-   char *msg1 = "Checking news via active file ...";
-   char *msg2 = "Checking news ...";
-   
-   if (Slrn_List_Active_File) slrn_message_now (msg1);
-   else slrn_message_now (msg2);
+   slrn_message_now ("Checking news%s ...",
+		     Slrn_List_Active_File ? " via active file" : "");
    
    if (create_flag)
      {
@@ -2846,7 +2936,10 @@ static char *group_display_format_cb (char ch, void *data, int *len, int *color)
 	     *len = 1;
 	  }
 	else
-	  *len = sprintf (buf, "%d", g->range.max); /* safe */
+#ifdef HAVE_ANSI_SPRINTF
+	  *len =
+#endif
+	  sprintf (buf, "%d", g->range.max); /* safe */
 	break;
       case 'l':
 	if (g->range.min == -1)
@@ -2855,7 +2948,10 @@ static char *group_display_format_cb (char ch, void *data, int *len, int *color)
 	     *len = 1;
 	  }
 	else
-	  *len = sprintf (buf, "%d", g->range.min); /* safe */
+#ifdef HAVE_ANSI_SPRINTF
+	  *len =
+#endif
+	  sprintf (buf, "%d", g->range.min); /* safe */
 	break;
       case 'n':
 	retval = g->group_name;
@@ -2868,10 +2964,16 @@ static char *group_display_format_cb (char ch, void *data, int *len, int *color)
 	     *len = 1;
 	  }
 	else
-	  *len = sprintf (buf, "%d", g->range.max - g->range.min + 1); /* safe */
+#ifdef HAVE_ANSI_SPRINTF
+	  *len =
+#endif
+	  sprintf (buf, "%d", g->range.max - g->range.min + 1); /* safe */
 	break;
       case 'u':
-	*len = sprintf (buf, "%d", g->unread); /* safe */
+#ifdef HAVE_ANSI_SPRINTF
+	*len =
+#endif
+	sprintf (buf, "%d", g->unread); /* safe */
 	break;
      }
    
