@@ -15,14 +15,17 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 extern int slrn_select_group_mode (void);
 extern void slrn_group_quit (void);
+extern int slrn_add_group (char *);
 extern void slrn_select_next_group (void);
 extern void slrn_select_prev_group (void);
 extern int slrn_group_select_group (void);
+extern void slrn_list_all_groups (int);
+extern void slrn_hide_groups (int);
 extern int slrn_read_newsrc (int);
 extern int slrn_write_newsrc (void);
 extern void slrn_mark_article_as_read (char *, long);
@@ -35,7 +38,7 @@ extern SLKeyMap_List_Type *Slrn_Group_Keymap;
 extern void slrn_post_cmd (void);
 extern unsigned int slrn_group_up_n (unsigned int);
 extern unsigned int slrn_group_down_n (unsigned int);
-extern int slrn_group_search (char *);
+extern int slrn_group_search (char *, int);
 extern void slrn_catchup_group (void);
 extern void slrn_uncatchup_group (void);
 extern void slrn_hide_current_group (void);
