@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -45,16 +45,15 @@
 #include "art.h"
 #include "misc.h"
 #include "startup.h"
+#include "util.h"
 #include "server.h"
 #include "menu.h"
 #include "interp.h"
-#include "util.h"
 #include "print.h"
 #include "score.h"
 #include "snprintf.h"
-#if SLRN_HAS_MIME 
-# include "mime.h"
-#endif
+#include "mime.h"
+#include "version.h"
 
 /*}}}*/
 
@@ -193,6 +192,23 @@ static void select_list_box (void)
    free_argv_list (argv, argc);
 }
 
+static int popup_window (void)
+{
+   char *title = NULL, *text = NULL;
+   int retval = 0;
+   
+   if ((-1 == SLpop_string (&text)) ||
+       (-1 == SLpop_string (&title)))
+     goto free_and_return;
+   
+   retval = slrn_popup_win_mode (title, text);
+   
+   free_and_return:
+   slrn_free (title);
+   slrn_free (text);
+   return retval;
+}
+
 static int get_yesno_cancel (char *prompt)
 {
    return slrn_get_yesno_cancel ("%s", prompt);
@@ -337,9 +353,11 @@ static char *interp_setlocale (int *category, char *locale) /*{{{*/
 
 /*}}}*/
 
-#if SLANG_VERSION >= 10400
 static void set_utf8_conversion_table (void) /*{{{*/
 {
+#if SLANG_VERSION < 10400
+   slrn_error ("To use this feature, please update s-lang and recompile slrn.");
+#else
 # if ! SLRN_HAS_MIME
    slrn_error ("This version of slrn lacks MIME support.");
 # else
@@ -401,9 +419,9 @@ static void set_utf8_conversion_table (void) /*{{{*/
    free_and_return:
    SLang_free_array (at);
 # endif /* SLRN_HAS_MIME */
+#endif /* SLANG_VERSION */
 }
 /*}}}*/
-#endif /* SLANG_VERSION */
 
 /*}}}*/
 
@@ -430,15 +448,25 @@ static void definekey (char *fun, char *key, char *map) /*{{{*/
 {
    SLKeyMap_List_Type *kmap;
    
+   if (!strcmp (map, "readline"))
+     map = "ReadLine";
+   
    if (NULL == (kmap = SLang_find_keymap (map)))
      {
-	error ("definekey: no such keymap.");
+	slrn_error ("definekey: no such keymap.");
 	return;
      }
    
-   if (0 != SLang_define_key (key, fun, kmap))
+   if ((kmap == Slrn_Keymap_RLI->keymap) &&
+       (NULL == SLang_find_key_function(fun, kmap)))
      {
+	SLKeymap_Function_Type *tmp = kmap->functions;
+	kmap->functions = Slrn_Custom_Readline_Functions;
+	SLang_define_key (key, fun, kmap);
+	kmap->functions = tmp;
      }
+   else
+     SLang_define_key (key, fun, kmap);
 }
 
 /*}}}*/
@@ -458,7 +486,9 @@ static void undefinekey (char *key, char *map) /*{{{*/
 
 /*}}}*/
 
-static void generic_read_mini (int no_echo, char *prompt, char *dfl, char *init) /*{{{*/
+#define PROMPT_NO_ECHO	1
+#define PROMPT_FILENAME	2
+static void generic_read_mini (int mode, char *prompt, char *dfl, char *init) /*{{{*/
 {
    char str[256];
    int ret;
@@ -466,9 +496,11 @@ static void generic_read_mini (int no_echo, char *prompt, char *dfl, char *init)
    strncpy (str, init, sizeof (str));
    str[sizeof(str) - 1] = 0;
    
-   if (no_echo) 
+   if (mode == PROMPT_NO_ECHO)
      ret = slrn_read_input_no_echo (prompt, dfl, str, 0, 0);
-   else 
+   else if (mode == PROMPT_FILENAME)
+     ret = slrn_read_filename (prompt, dfl, str, 0, 0);
+   else
      ret = slrn_read_input (prompt, dfl, str, 0, 0);
    
    if (-1 == ret)
@@ -488,7 +520,14 @@ static void read_mini (char *prompt, char *dfl, char *init) /*{{{*/
 /*}}}*/
 static void read_mini_no_echo (char *prompt, char *dfl, char *init) /*{{{*/
 {
-   generic_read_mini (1, prompt, dfl, init);
+   generic_read_mini (PROMPT_NO_ECHO, prompt, dfl, init);
+}
+
+/*}}}*/
+
+static void read_mini_filename (char *prompt, char *dfl, char *init) /*{{{*/
+{
+   generic_read_mini (PROMPT_FILENAME, prompt, dfl, init);
 }
 
 /*}}}*/
@@ -1013,6 +1052,9 @@ static int group_down_n (int *np)
 {
    int n = *np;
    
+   if (-1 == check_group_mode ())
+     return 0;
+   
    if (n < 0) return slrn_group_up_n (-n);
    return slrn_group_down_n (n);
 }
@@ -1021,10 +1063,20 @@ static int group_up_n (int *np)
 {
    int n = *np;
    
+   if (-1 == check_group_mode ())
+     return 0;
+   
    if (n < 0) return slrn_group_down_n (-n);
    return slrn_group_up_n (n);
 }
 
+static int group_search (char *str)
+{
+   if (-1 == check_group_mode ())
+     return 0;
+   return slrn_group_search (str, 1);
+}
+
 static char *current_group_name (void)
 {
    if (Slrn_Group_Current_Group == NULL)
@@ -1035,6 +1087,8 @@ static char *current_group_name (void)
 
 static int get_group_unread_count (void)
 {
+   if (is_article_mode ())
+     return slrn_art_get_unread ();
    if (Slrn_Group_Current_Group == NULL) return 0;
    return Slrn_Group_Current_Group->unread;
 }
@@ -1067,6 +1121,12 @@ static void set_group_flags (int *flagsp)
    Slrn_Group_Current_Group->flags |= flags;
 }
 
+static void hide_current_group (void)
+{
+   if (0 == check_group_mode ())
+     slrn_hide_current_group ();
+}
+
 /*}}}*/
 
 static void set_color (char *obj, char *fg, char *bg)
@@ -1126,6 +1186,11 @@ static void set_header_display_format (int *num, char *fmt)
    (void) slrn_set_header_format (*num, fmt);
 }
 
+static void set_group_display_format (int *num, char *fmt)
+{
+   (void) slrn_set_group_format (*num, fmt);
+}
+
 static void print_file (char *file)
 {
    (void) slrn_print_file (file);
@@ -1187,6 +1252,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("close_log_file", close_log_file, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("log_message", log_message, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_IS("set_header_display_format", set_header_display_format, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_IS("set_group_display_format", set_group_display_format, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_prefix_arg", get_prefix_arg, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("reset_prefix_arg", reset_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("locate_header_by_msgid", locate_header_by_msgid, SLANG_INT_TYPE),
@@ -1213,7 +1279,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("getkey", getkey, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("goto_num_tagged_header", slrn_goto_num_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("group_down_n", group_down_n, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_S("group_search", slrn_group_search, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_S("group_search", group_search, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("group_unread", get_group_unread_count, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("group_up_n", group_up_n, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("header_down", header_down, SLANG_INT_TYPE),
@@ -1228,6 +1294,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_S("make_home_filename", make_home_filename, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("next_tagged_header", slrn_next_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("pipe_article", pipe_article_cmd, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("popup_window", popup_window, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("prev_tagged_header", slrn_prev_tagged_header, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("quit", quit, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("re_bsearch_author", re_author_search_backward, SLANG_INT_TYPE),
@@ -1237,6 +1304,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_S("re_search_article", re_search_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_SSS("read_mini", read_mini, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_no_echo", read_mini_no_echo, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_SSS("read_mini_filename", read_mini_filename, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
    MAKE_INTRINSIC_S("search_article", search_article, SLANG_INT_TYPE),
@@ -1247,6 +1315,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_I("set_article_window_size", set_article_window_size, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("set_color", set_color, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_group_flags", set_group_flags, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("hide_current_group", hide_current_group, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_header_flags", set_header_flags, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_header_score", set_header_score, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_input_chars", set_input_chars, SLANG_VOID_TYPE),
@@ -1254,9 +1323,9 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SI("set_integer_variable", set_integer_variable, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("set_prefix_argument", set_prefix_arg, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("set_string_variable", set_string_variable, SLANG_VOID_TYPE),
-#if SLANG_VERSION >= 10400
    MAKE_INTRINSIC_0("set_utf8_conversion_table", set_utf8_conversion_table, SLANG_VOID_TYPE),
-#endif
+   MAKE_INTRINSIC_0("get_group_order", slrn_intr_get_group_order, SLANG_VOID_TYPE),
+   MAKE_INTRINSIC_0("set_group_order", slrn_intr_set_group_order, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_IS("setlocale", interp_setlocale, SLANG_STRING_TYPE),
    MAKE_INTRINSIC_0("sort_by_sorting_method", sort_articles, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("thread_size", thread_size, SLANG_INT_TYPE),
@@ -1282,6 +1351,8 @@ static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
    MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("SCREEN_HEIGHT", &SLtt_Screen_Rows, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("SCREEN_WIDTH", &SLtt_Screen_Cols, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version", &Slrn_Version_Number, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version, SLANG_STRING_TYPE, 1),
 #ifdef HAVE_SETLOCALE
 # ifdef LC_CTYPE
    MAKE_VARIABLE("LC_CTYPE", &Interp_Lc_Ctype, SLANG_INT_TYPE, 1),
