@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -386,12 +386,13 @@ static unsigned int Select_List_Window_Col;
 static unsigned int Select_List_Window_Ncols;
 static int Select_List_Active_Color = SELECT_COLOR;
 static char *Select_List_Title = ".";
+static int Is_Popup_Window; /* set to 2 if full text is visible */
 
 static void draw_select_list (void)
 {
    unsigned int r;
    Select_List_Type *l;
-   int row, column;
+   int row, column, top_showing = 0, bot_showing = 0;
    unsigned int num_rows, num_columns;
 
    if (Select_Window == NULL)
@@ -413,7 +414,28 @@ static void draw_select_list (void)
    slrn_set_color (FRAME_COLOR);
    SLsmg_draw_box (row, column, num_rows + 2, num_columns + 2);
    SLsmg_gotorc (row + num_rows + 1, column + num_columns - 10);
-   SLsmg_printf ("[%u/%u]", Select_Window->line_num, Select_Window->num_lines);
+   if (Is_Popup_Window)
+     {
+	unsigned int bot_number;
+	
+	bot_number = Select_Window->line_num +
+	  (Select_Window->nrows - Select_Window->window_row) - 1;
+	bot_showing = ((Select_Window->bot_window_line == NULL)
+		       || (Select_Window->num_lines == bot_number));
+	top_showing = (Select_Window->line_num == Select_Window->window_row + 1);
+	
+	if (top_showing)
+	  {
+	     SLsmg_write_string (bot_showing ? "[All]" : "[Top]");
+	     Is_Popup_Window = bot_showing ? 2 : 1;
+	  }
+	else if (bot_showing)
+	  SLsmg_write_string ("[Bot]");
+	else
+	  SLsmg_printf ("[%u/%u]", Select_Window->line_num, Select_Window->num_lines);
+     }
+   else
+     SLsmg_printf ("[%u/%u]", Select_Window->line_num, Select_Window->num_lines);
 
    if (Select_List_Title != NULL)
      {
@@ -433,7 +455,8 @@ static void draw_select_list (void)
 	if (l == NULL) data = NULL;
 	else 
 	  {
-	     if (l == (Select_List_Type *) Select_Window->current_line)
+	     if (!Is_Popup_Window &&
+		 (l == (Select_List_Type *) Select_Window->current_line))
 	       slrn_set_color (Select_List_Active_Color);
 
 	     data = l->data;
@@ -444,8 +467,18 @@ static void draw_select_list (void)
 	slrn_set_color (BOX_COLOR);
      }
 
-   slrn_message ("Use UP/DOWN to move, RETURN to select, Ctrl-G to cancel");
-   SLsmg_gotorc (row + Select_Window->window_row, column);
+   if (Is_Popup_Window)
+     {
+	if (top_showing && bot_showing)
+	  slrn_message ("Press any key to close the window.");
+	else
+	  slrn_message ("Use UP/DOWN to move, any other key to close the window.");
+     }
+   else
+     {
+	slrn_message ("Use UP/DOWN to move, RETURN to select, Ctrl-G to cancel");
+	SLsmg_gotorc (row + Select_Window->window_row, column);
+     }
    slrn_pop_suspension ();
 }
 
@@ -491,37 +524,71 @@ static Slrn_Mode_Type Select_List_Mode_Cap =
 
 static int Select_List_Quit;
 
+static void sl_cancel (void)
+{
+   Select_List_Quit = -1;
+}
+
 static void sl_up (void)
 {
-   if (0 == SLscroll_prev_n (Select_Window, 1))
+   if (Is_Popup_Window == 2)
+     {
+	sl_cancel ();
+	return;
+     }
+   if ((0 == SLscroll_prev_n (Select_Window, 1)) &&
+       (0 == Is_Popup_Window))
      {
 	while (0 != SLscroll_next_n (Select_Window, 1000))
 	  ;
      }
+   if (Is_Popup_Window)
+     Select_Window->top_window_line = Select_Window->current_line;
 }
 
 static void sl_pageup (void)
 {
-   SLscroll_pageup (Select_Window);
+   if (Is_Popup_Window == 2)
+     {
+	sl_cancel ();
+	return;
+     }
+   if (Is_Popup_Window)
+     {
+	SLscroll_prev_n (Select_Window, Select_Window->nrows - 1);
+	Select_Window->top_window_line = Select_Window->current_line;
+     }
+   else
+     SLscroll_pageup (Select_Window);
 }
 
 static void sl_pagedn (void)
 {
+   if (Is_Popup_Window == 2)
+     {
+	sl_cancel ();
+	return;
+     }
    SLscroll_pagedown (Select_Window);
+   if (Is_Popup_Window)
+     Select_Window->top_window_line = Select_Window->current_line;
 }
 
 static void sl_down (void)
 {
-   if (0 == SLscroll_next_n (Select_Window, 1))
+   if (Is_Popup_Window == 2)
+     {
+	sl_cancel ();
+	return;
+     }
+   if ((0 == SLscroll_next_n (Select_Window, 1)) &&
+       (0 == Is_Popup_Window))
      {
 	while (0 != SLscroll_prev_n (Select_Window, 1000))
 	  ;
      }
-}
-
-static void sl_cancel (void)
-{
-   Select_List_Quit = -1;
+   if (Is_Popup_Window)
+     Select_Window->top_window_line = Select_Window->current_line;
 }
 
 static void sl_select (void)
@@ -569,10 +636,11 @@ static void sl_mouse (void)
    
 static SLKeyMap_List_Type *Select_List_Keymap;
 
-static int init_select_list_mode (void)
+static int init_select_list_mode (int select_list)
 {
    slrn_free ((char *) Select_Window);
    Select_Window = (SLscroll_Window_Type *) slrn_malloc (sizeof (SLscroll_Window_Type), 1, 1);
+   Is_Popup_Window = !select_list;
    if (Select_Window == NULL)
      return -1;
    Select_Window->nrows = 5;
@@ -593,10 +661,6 @@ static int init_select_list_mode (void)
    SLkm_define_key  ("\xE0I", (FVOID_STAR) sl_pageup, Select_List_Keymap);
    SLkm_define_key  ("^@Q", (FVOID_STAR) sl_pagedn, Select_List_Keymap);
    SLkm_define_key  ("\xE0Q", (FVOID_STAR) sl_pagedn, Select_List_Keymap);
-   SLkm_define_key  ("^@M", (FVOID_STAR) sl_right, Select_List_Keymap);
-   SLkm_define_key  ("\xE0M", (FVOID_STAR) sl_right, Select_List_Keymap);
-   SLkm_define_key  ("^@K", (FVOID_STAR) sl_left, Select_List_Keymap);
-   SLkm_define_key  ("\xE0K", (FVOID_STAR) sl_left, Select_List_Keymap);
 #else
    SLkm_define_key  ("\033[5~", (FVOID_STAR) sl_pageup, Select_List_Keymap);
    SLkm_define_key  ("\033[6~", (FVOID_STAR) sl_pagedn, Select_List_Keymap);
@@ -604,14 +668,7 @@ static int init_select_list_mode (void)
    SLkm_define_key  ("\033OA", (FVOID_STAR) sl_up, Select_List_Keymap);
    SLkm_define_key  ("\033[B", (FVOID_STAR) sl_down, Select_List_Keymap);
    SLkm_define_key  ("\033OB", (FVOID_STAR) sl_down, Select_List_Keymap);
-   SLkm_define_key  ("\033[C", (FVOID_STAR) sl_right, Select_List_Keymap);
-   SLkm_define_key  ("\033OC", (FVOID_STAR) sl_right, Select_List_Keymap);
-   SLkm_define_key  ("\033OD", (FVOID_STAR) sl_left, Select_List_Keymap);
-   SLkm_define_key  ("\033[D", (FVOID_STAR) sl_left, Select_List_Keymap);
-
 # ifdef __unix__
-   SLkm_define_key  ("^(kr)", (FVOID_STAR) sl_right, Select_List_Keymap);
-   SLkm_define_key  ("^(kl)", (FVOID_STAR) sl_left, Select_List_Keymap);
    SLkm_define_key  ("^(kd)", (FVOID_STAR) sl_down, Select_List_Keymap);
    SLkm_define_key  ("^(ku)", (FVOID_STAR) sl_up, Select_List_Keymap);
 # endif
@@ -620,12 +677,30 @@ static int init_select_list_mode (void)
    SLkm_define_key  ("k", (FVOID_STAR) sl_up, Select_List_Keymap);
    SLkm_define_key  ("j", (FVOID_STAR) sl_down, Select_List_Keymap);
    
-   SLkm_define_key  ("\r", (FVOID_STAR) sl_select, Select_List_Keymap);
-   SLkm_define_key  ("^G", (FVOID_STAR) sl_cancel, Select_List_Keymap);
-   
-   SLkm_define_key  ("\033[M\040", (FVOID_STAR) sl_mouse, Select_List_Keymap);
-   SLkm_define_key  ("\033[M\041", (FVOID_STAR) sl_mouse, Select_List_Keymap);
-   SLkm_define_key  ("\033[M\042", (FVOID_STAR) sl_mouse, Select_List_Keymap);
+   if (select_list)
+     {
+#if defined(IBMPC_SYSTEM)
+	SLkm_define_key  ("^@M", (FVOID_STAR) sl_right, Select_List_Keymap);
+	SLkm_define_key  ("\xE0M", (FVOID_STAR) sl_right, Select_List_Keymap);
+	SLkm_define_key  ("^@K", (FVOID_STAR) sl_left, Select_List_Keymap);
+	SLkm_define_key  ("\xE0K", (FVOID_STAR) sl_left, Select_List_Keymap);
+#else
+	SLkm_define_key  ("\033[C", (FVOID_STAR) sl_right, Select_List_Keymap);
+	SLkm_define_key  ("\033OC", (FVOID_STAR) sl_right, Select_List_Keymap);
+	SLkm_define_key  ("\033OD", (FVOID_STAR) sl_left, Select_List_Keymap);
+	SLkm_define_key  ("\033[D", (FVOID_STAR) sl_left, Select_List_Keymap);
+# ifdef __unix__
+	SLkm_define_key  ("^(kr)", (FVOID_STAR) sl_right, Select_List_Keymap);
+	SLkm_define_key  ("^(kl)", (FVOID_STAR) sl_left, Select_List_Keymap);
+# endif
+#endif
+	SLkm_define_key  ("\r", (FVOID_STAR) sl_select, Select_List_Keymap);
+	SLkm_define_key  ("^G", (FVOID_STAR) sl_cancel, Select_List_Keymap);
+	
+	SLkm_define_key  ("\033[M\040", (FVOID_STAR) sl_mouse, Select_List_Keymap);
+	SLkm_define_key  ("\033[M\041", (FVOID_STAR) sl_mouse, Select_List_Keymap);
+	SLkm_define_key  ("\033[M\042", (FVOID_STAR) sl_mouse, Select_List_Keymap);
+     }
    
    if (SLang_Error)
      return -1;
@@ -646,7 +721,7 @@ int slrn_select_list_mode (char *title,
    if (argv == 0)
      return -1;
 
-   if (-1 == init_select_list_mode ())
+   if (-1 == init_select_list_mode (1))
      return -1;
    
    Select_List_Title = title;
@@ -718,8 +793,91 @@ int slrn_select_list_mode (char *title,
    return (int) active_num;
 }
 
-   
+/* This function will modify "text"! */
+int slrn_popup_win_mode (char *title, char *text)
+{
+   unsigned int num_lines = 0;
+   Select_List_Type *root, *curr, *last;
 
+   if ((text == NULL) || (*text == 0) ||
+       (-1 == init_select_list_mode (0)))
+     return -1;
+   
+   Select_List_Title = title;
+
+   if (title == NULL) 
+     Select_List_Window_Ncols = 1;
+   else 
+     Select_List_Window_Ncols = 2 + strlen (title);
+
+   root = last = NULL;
+   
+   do
+     {
+	unsigned int len;
+	char *newline;
+	
+	if (NULL == (curr = (Select_List_Type *) slrn_malloc (sizeof (Select_List_Type), 1, 1)))
+	  {
+	     free_select_list (root);
+	     return -1;
+	  }
+	
+	if (root == NULL)
+	  root = curr;
+	else
+	  {
+	     last->next = curr;
+	     curr->prev = last;
+	  }
+	last = curr;
+	
+	curr->data = text;
+	if (NULL != (newline = strchr (text, '\n')))
+	  {
+	     *newline++ = 0;
+	     if (*newline == 0)
+	       newline = NULL;
+	  }
+	len = strlen (text);
+	if (len > Select_List_Window_Ncols)
+	  Select_List_Window_Ncols = len;
+	text = newline;
+	num_lines++;
+     }
+   while (text != NULL);
+   
+   Select_Window->num_lines = num_lines;
+   Select_Window->nrows = num_lines;
+   Select_Window->current_line = Select_Window->lines = (SLscroll_Type *) root;
+   select_list_winch (0, 0);
+   SLscroll_find_line_num (Select_Window);
+   Select_Window->top_window_line = Select_Window->current_line;
+   
+   slrn_push_mode (&Select_List_Mode_Cap);
+   Select_List_Quit = 0;
+   while (Select_List_Quit == 0)
+     {
+	SLang_Key_Type *key;
+	
+	slrn_update_screen ();
+	key = SLang_do_key (Select_List_Keymap, (int (*)(void)) SLang_getkey);
+	if ((key == NULL) 
+	    || (key->type == SLKEY_F_INTERPRET))
+	  Select_List_Quit = 1;
+	else
+	  key->f.f ();
+     }
+   free_select_list (root);
+   slrn_pop_mode ();
+   
+   slrn_message ("");
+   
+   if (Select_List_Quit == -1)
+     return -1;
+   
+   return SLang_Last_Key_Char;;
+}
 
 static int file_strcmp (char **ap, char **bp)
 {
@@ -738,17 +896,6 @@ static int file_strcmp (char **ap, char **bp)
    return strcmp (a, b);
 }
 
-
-static void free_dir_list (unsigned int argc, char **argv)
-{
-   while (argc)
-     {
-	argc--;
-	slrn_free (argv[argc]);
-     }
-}
-
-
 #define MAX_DIR_FILES 1024
 
 static char *browse_dir (char *dir)
@@ -802,7 +949,7 @@ static char *browse_dir (char *dir)
 	len = de->name_len;
 	if (NULL == (argv[argc] = slrn_malloc (len + 2, 0, 1)))
 	  {
-	     free_dir_list (argc, argv);
+	     slrn_free_argc_argv_list (argc, argv);
 	     return NULL;
 	  }
 	strcpy (argv[argc], de->name); /* safe */
@@ -825,13 +972,13 @@ static char *browse_dir (char *dir)
 						((argc > 2) ? 2 : 0),
 						NULL)))
      {
-	free_dir_list (argc, argv);
+	slrn_free_argc_argv_list (argc, argv);
 	return NULL;
      }
    
    dir = argv[selected];
    argv[selected] = NULL;
-   free_dir_list (argc, argv);
+   slrn_free_argc_argv_list (argc, argv);
    return dir;
 }
 
