@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -42,19 +42,21 @@
 #endif
 
 #include "nntpcodes.h"
-#include "nntplib.h"
 #include "util.h"
+#include "nntplib.h"
 #include "ttymsg.h"
 #include "snprintf.h"
 
 void (*NNTP_Connection_Lost_Hook) (NNTP_Type *);
 int (*NNTP_Authorization_Hook) (char *, char **, char **);
+int NNTP_Try_Authentication = 0;
 
-FILE *NNTP_Debug_Fp;
 int Slrn_Broken_Xref;
 
 static int _nntp_connect_server (NNTP_Type *);
 
+extern FILE *Slrn_Debug_Fp; /* we need this here when compiling slrnpull */
+
 static NNTP_Type *nntp_allocate_nntp (void)
 {
    static NNTP_Type nn;
@@ -66,7 +68,6 @@ static NNTP_Type *nntp_allocate_nntp (void)
    s->can_xover = -1;
    s->can_xhdr = -1;
    s->can_xpat = -1;
-   s->can_xgtitle = -1;
 
    return s;
 }
@@ -105,8 +106,8 @@ int nntp_fgets_server (NNTP_Type *s, char *buf, unsigned int len)
 	return -1;
      }
 
-   if (NNTP_Debug_Fp != NULL)
-     fprintf (NNTP_Debug_Fp, "<%s", buf);
+   if (Slrn_Debug_Fp != NULL)
+     fprintf (Slrn_Debug_Fp, "<%s", buf);
 
    return 0;
 }
@@ -115,17 +116,17 @@ int nntp_fputs_server (NNTP_Type *s, char *buf)
 {
    if ((s == NULL) || (s->init_state == 0))
      return -1;
-
-   if (NNTP_Debug_Fp != NULL)
-     fprintf (NNTP_Debug_Fp, ">%s\n", buf);
-
+   
    if (-1 == sltcp_fputs (s->tcp, buf))
      {
 	(void) nntp_disconnect_server (s);
 	(void) check_connect_lost_hook (s);
 	return -1;
      }
-
+   
+   if (Slrn_Debug_Fp != NULL)
+     fprintf (Slrn_Debug_Fp, ">%s\n", buf);
+   
    return 0;
 }
 
@@ -133,23 +134,23 @@ int nntp_write_server (NNTP_Type *s, char *buf, unsigned int n)
 {
    if ((s == NULL) || (s->init_state == 0))
      return -1;
-
-   if (NNTP_Debug_Fp != NULL)
-     {
-	unsigned int i;
-	fputc ('>', NNTP_Debug_Fp);
-	for (i = 0; i < n; i++)
-	  fputc (buf[i], NNTP_Debug_Fp);
-	fputc ('\n', NNTP_Debug_Fp);
-     }
-
+   
    if (n != sltcp_write (s->tcp, buf, n))
      {
 	(void) nntp_disconnect_server (s);
 	(void) check_connect_lost_hook (s);
 	return -1;
      }
-
+   
+   if (Slrn_Debug_Fp != NULL)
+     {
+	unsigned int i;
+	fputc ('>', Slrn_Debug_Fp);
+	for (i = 0; i < n; i++)
+	  fputc (buf[i], Slrn_Debug_Fp);
+	fputc ('\n', Slrn_Debug_Fp);
+     }
+   
    return 0;
 }
 
@@ -203,6 +204,9 @@ static int _nntp_try_parse_timeout (char *str)
     *
     * Here the idea is to look for 'time' and then 'out'.
     */
+   /* Some servers now have NLS, so this might fail.
+    * I'm trying to handle that case in nntp_server_cmd.
+    */
    static SLRegexp_Type re;
    unsigned char compiled_pattern_buf[256];
 
@@ -267,7 +271,7 @@ static int _nntp_reconnect (NNTP_Type *s)
 	s->flags |= NNTP_RECONNECT_OK;
 	return -1;
      }
-
+   
    if ((s->group_name[0] != 0)
        && (-1 == nntp_server_vcmd (s, "GROUP %s", s->group_name)))
      {
@@ -305,21 +309,40 @@ int nntp_start_server_cmd (NNTP_Type *s, char *cmd)
 
 int nntp_server_cmd (NNTP_Type *s, char *cmd)
 {
-   int max_tries = 3;
+   int max_tries = 3, tried_auth;
 
    do
      {
 	int code;
+	tried_auth = 0;
 
 	if (-1 == nntp_start_server_cmd (s, cmd))
 	  return -1;
 
 	if (-1 != (code = nntp_get_server_response (s)))
-	  return code;
+	  {
+	     if ((code == ERR_NOAUTH) && NNTP_Try_Authentication)
+	       {
+		  NNTP_Try_Authentication = 0;
+		  if (-1 == nntp_authorization (s))
+		    return -1;
+		  tried_auth = 1;
+		  continue;
+	       }
+	     else if ((code == ERR_FAULT) && (max_tries == 3))
+	       /* might be a timeout we didn't recognize */
+	       {
+		  nntp_disconnect_server (s);
+		  (void) check_connect_lost_hook (s);
+		  max_tries = 1;
+		  continue;
+	       }
+	     return code;
+	  }
 
 	max_tries--;
      }
-   while (max_tries && (s->flags & NNTP_RECONNECT_OK));
+   while ((max_tries && (s->flags & NNTP_RECONNECT_OK)) || tried_auth);
 
    return -1;
 }
@@ -363,6 +386,10 @@ int nntp_close_server (NNTP_Type *s)
     * of the *server_cmd functions.
     */
    (void) nntp_puts_server (s, "QUIT");
+   
+   if (Slrn_Debug_Fp != NULL)
+     fputs ("!Closing the server connection.\n", Slrn_Debug_Fp);
+   
    (void) sltcp_close (s->tcp);
 
    s->tcp = NULL;
@@ -377,33 +404,46 @@ int nntp_authorization (NNTP_Type *s)
 {
    char *name, *pass;
 
-   if (NNTP_Authorization_Hook == NULL)
-     return 0;
-
-   if (-1 == (*NNTP_Authorization_Hook) (s->host, &name, &pass))
-     return -1;
-
-   if ((name == NULL) || (pass == NULL))
-     return 0;
+   if ((NNTP_Authorization_Hook == NULL) ||
+       (-1 == (*NNTP_Authorization_Hook) (s->host, &name, &pass)) ||
+       ((name == NULL) || (pass == NULL)))
+     {
+	slrn_exit_error ("Authorization needed, but could not determine username / password.");
+	return -1;
+     }
 
    slrn_message_now ("Authenticating %s ...", name);
 
    if (-1 == nntp_server_vcmd (s, "AUTHINFO USER %s", name))
      return -1;
 
-   if (s->code != NEED_AUTHDATA)
+   if (s->code == OK_AUTH)
      return 0;
+   if (s->code != NEED_AUTHDATA)
+     return -1;
 
    switch (nntp_server_vcmd (s, "AUTHINFO PASS %s", pass))
      {
       case -1:
 	return -1;
+      case OK_AUTH:
+	if (s->can_post == 1)
+	  break;
+	/* This is the only obvious way to find out whether we are
+	 * able to post after successful authentication. */
+	if (-1 == nntp_post_cmd (s))
+	  return -1;
+	if (s->code == CONT_POST)
+	  {
+	     nntp_end_post (s);
+	     s->can_post = 1;
+	     break;
+	  }
+	break;
       case ERR_ACCESS:
+      default:
 	_nntp_error_response (s, "Authorization failed.");
 	return -1;
-      case OK_AUTH:
-	s->can_post = 1;
-	break;
      }
    return 0;
 }
@@ -414,12 +454,15 @@ static int _nntp_connect_server (NNTP_Type *s)
      sltcp_close (s->tcp);
 
    slrn_message_now ("Connecting to host %s ...", s->host);
+   if (Slrn_Debug_Fp != NULL)
+     fputs ("!Connecting to server...\n", Slrn_Debug_Fp);
 
    if (NULL == (s->tcp = sltcp_open_connection (s->host, s->port, s->use_ssh)))
      return -1;
 
    s->init_state = 1;
    s->number_bytes_received = 0;
+   NNTP_Try_Authentication = 1;
    
    /* Read logon message. */
    switch (nntp_get_server_response (s))
@@ -437,9 +480,11 @@ static int _nntp_connect_server (NNTP_Type *s)
      }
 
    if ((-1 == nntp_server_cmd (s, "MODE READER"))
-       || (ERR_ACCESS == s->code)
-       || (-1 == nntp_authorization (s)))
+       || (ERR_ACCESS == s->code))
      goto failed;
+   
+   if (s->code == OK_NOPOST)
+     s->can_post = 0;
 
    slrn_message_now ("Connected to host.  Posting %sOk.",
 		     (s->can_post ? "" : "NOT "));
@@ -448,6 +493,9 @@ static int _nntp_connect_server (NNTP_Type *s)
    failed:
 
    _nntp_error_response (s, "Failed to initialize server");
+   if (Slrn_Debug_Fp != NULL)
+     fputs ("!Server connect failed.\n", Slrn_Debug_Fp);
+   
    (void) sltcp_close (s->tcp);
    s->tcp = NULL;
    return -1;
@@ -516,6 +564,7 @@ char *nntp_get_server_name (void)
    fprintf (stderr, "Example: set NNTPSERVER=my.news.server\r\n");
 # else
    fprintf (stderr, "Example (csh): setenv NNTPSERVER my.news.server\r\n");
+   fprintf (stderr, "Example (sh) : NNTPSERVER='my.news.server' && export NNTPSERVER\r\n");
 # endif
 #endif
    return NULL;
@@ -572,11 +621,6 @@ NNTP_Type *nntp_open_server (char *host, int port)
 {
    NNTP_Type *s;
 
-#if 0
-   if (NNTP_Debug_Fp == NULL)
-     NNTP_Debug_Fp = fopen ("nntp.log", "w");
-#endif
-
    if (host == NULL)
      {
 	host = nntp_get_server_name ();
@@ -651,7 +695,10 @@ int nntp_check_connection (NNTP_Type *s)
 void nntp_disconnect_server (NNTP_Type *s)
 {
    if (s == NULL) return;
-
+   
+   if (Slrn_Debug_Fp != NULL)
+     fputs ("!Disconnecting from server.\n", Slrn_Debug_Fp);
+   
    sltcp_close_socket (s->tcp);
 }
 
@@ -670,39 +717,39 @@ static int _nntp_probe_server (NNTP_Type *s, char *cmd)
 
 int nntp_has_cmd (NNTP_Type *s, char *cmd)
 {
-   /* DNEWS, in my opinion does not handle XHDR probing properly.
-    * Sigh.
-    */
-   if (!strcmp (cmd, "XHDR"))
+   char buf [NNTP_BUFFER_SIZE];
+   if (!strncmp (cmd, "XHDR", 4))
      {
 	int ret;
 
 	if (s->can_xhdr != -1)
 	  return s->can_xhdr;
 
-	if (-1 == (ret = nntp_server_cmd (s, cmd)))
-	  return -1;
+	ret = nntp_server_cmd (s, cmd);
 
 	if (ret == ERR_COMMAND)
+	  s->can_xhdr = 0;
+	else if ((ret == OK_HEAD) &&
+		 (1 == nntp_read_line (s, buf, sizeof (buf))))
 	  {
-	     s->can_xhdr = 0;
-	     return 0;
+	     char *p = buf;
+	     nntp_discard_output (s);
+	     while (*p && (*p != ' ')) p++;
+	     while (*p == ' ') p++;
+	     if (!strcmp (p, "(none)"))
+	       s->can_xhdr = 0;
+	     else
+	       s->can_xhdr = 1;
 	  }
-
-	s->can_xhdr = 1;
-
-	if (ret == OK_HEAD)  /* DNEWS bug?? */
-	  (void) nntp_discard_output (s);
-
-	return 1;
+	else if ((ret == ERR_CMDSYN) && (strlen (cmd) == 4))
+	  s->can_xhdr = 1;
+	
+	return s->can_xhdr;
      }
 
    if (!strcmp (cmd, "XPAT"))
      return PROBE_XCMD(s, s->can_xpat, cmd);
 
-   if (!strcmp (cmd, "XGTITLE"))
-     return PROBE_XCMD(s, s->can_xgtitle, cmd);
-
    if (!strcmp (cmd, "XOVER"))
      return PROBE_XCMD(s, s->can_xover, cmd);
 
@@ -760,7 +807,6 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 {
    int estim;
    int min, max;
-   static int try_again = 1;
    
    switch (nntp_server_vcmd (s, "GROUP %s", name))
      {
@@ -778,21 +824,76 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 	slrn_strncpy (s->group_name, name, NNTP_MAX_GROUP_NAME_LEN);
 	break;
 	
-      case ERR_NOAUTH:
-	if (try_again && (-1 != nntp_authorization (s)))
-	  {
-	     try_again = 0;
-	     return nntp_select_group (s, name, minp, maxp);
-	  }
       case ERR_ACCESS:
       default:
 	break;
      }
 
-   try_again = 1;
    return s->code;
 }
 
+int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
+{
+   int reconnect, i, max_tries = 3;
+   
+   start_over:
+   max_tries--;
+   reconnect = s->flags & NNTP_RECONNECT_OK;
+   s->flags &= ~NNTP_RECONNECT_OK;
+   /* disallow reconnect for the moment as we batch commands */
+   
+   i = 0;
+   while (i < n)
+     {
+	if (-1 == nntp_start_server_vcmd (s, "GROUP %s", gr[i].name))
+	  {
+	     s->flags |= reconnect;
+	     if ((max_tries == 0) || (-1 == _nntp_reconnect (s)))
+	       return -1;
+	     goto start_over;
+	  }
+	i++;
+     }
+   i = 0;
+   while (i < n)
+     {
+	int estim;
+	switch (nntp_get_server_response (s))
+	  {
+	   case -1:
+	     s->flags |= reconnect;
+	     if ((max_tries == 0) || (-1 == _nntp_reconnect (s)))
+	       return -1;
+	     goto start_over;
+	     
+	   case OK_GROUP:
+	     if (3 != sscanf (s->rspbuf + 4, "%d %d %d", &estim,
+			      &(gr[i].min), &(gr[i].max)))
+	       return -1;
+	     
+	     slrn_strncpy (s->group_name, gr[i].name, NNTP_MAX_GROUP_NAME_LEN);
+	     break;
+	     
+	   case ERR_FAULT:
+	     if (max_tries == 2) /* might be a timeout we didn't recognize */
+	       {
+		  nntp_disconnect_server (s);
+		  (void) check_connect_lost_hook (s);
+		  max_tries = 0;
+		  goto start_over;
+	       }
+	     /* Fall through */
+	     
+	   default:
+	     gr[i].min = -1;
+	     break;
+	  }
+	i++;
+     }
+   s->flags |= reconnect;
+   return 0;
+}
+
 int nntp_post_cmd (NNTP_Type *s)
 {
    return _nntp_num_or_msgid_cmd (s, "POST", -1, NULL);
@@ -814,28 +915,6 @@ int nntp_xpat_cmd (NNTP_Type *s, char *hdr, int rmin, int rmax, char *pat)
    return nntp_server_vcmd (s, "XPAT %s %d-%d *%s*", hdr, rmin, rmax, pat);
 }
 
-int nntp_xgtitle_cmd (NNTP_Type *s, char *pattern)
-{
-   if (s->can_xgtitle == 0)
-     return -1;
-
-   /* XGTITLE appears broken on some servers.  So, do not probe for it. */
-   slrn_message_now ("Sending Query to Server ...");
-
-   if (-1 == nntp_server_vcmd (s, "XGTITLE %s", pattern))
-     return -1;
-
-   if (s->code != OK_XGTITLE)
-     {
-	slrn_message ("Server does not support XGTITLE command.");
-	s->can_xgtitle = 0;
-	return ERR_COMMAND;
-     }
-
-   s->can_xgtitle = 1;
-   return OK_XGTITLE;
-}
-
 /* hdr should not include ':' */
 int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int buflen)
 {
@@ -913,14 +992,22 @@ int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int
    return status;
 }
 
-int nntp_list_newsgroups (NNTP_Type *s)
+int nntp_list (NNTP_Type *s, char* what)
 {
-   return _nntp_num_or_msgid_cmd (s, "LIST", -1, "NEWSGROUPS");
+   return nntp_server_vcmd (s, "LIST %s", what);
 }
 
-int nntp_list_active_cmd (NNTP_Type *s)
+int nntp_list_newsgroups (NNTP_Type *s)
 {
-   return _nntp_num_or_msgid_cmd (s, "LIST", -1, NULL);
+   return nntp_server_vcmd (s, "LIST NEWSGROUPS");
+}
+
+int nntp_list_active_cmd (NNTP_Type *s, char* pat)
+{
+   if (pat == NULL)
+     return nntp_server_cmd (s, "LIST");
+   else
+     return nntp_server_vcmd (s, "LIST ACTIVE %s", pat);
 }
 
 int nntp_listgroup (NNTP_Type *s, char *group)
