@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #ifndef SLRNPULL_CODE
@@ -39,9 +39,9 @@
 # include "slrn.h"
 # include "group.h"
 # include "misc.h"
+# include "util.h"
 # include "server.h"
 # include "hash.h"
-# include "util.h"
 #endif
 
 #include "art.h"
@@ -217,14 +217,7 @@ static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
              break;
 
            case SCORE_GENERIC:
-#ifdef SLRNPULL_CODE
-             s = slrn_get_extra_xover_header (srt->generic_keyword);
-#else
-	     if (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING)
-	       s = slrn_extract_add_header (h, srt->generic_keyword);
-	     else
-	       s = NULL;
-#endif
+	     s = slrn_extract_add_header (h, srt->generic_keyword);
              break;
 
            case SCORE_SUB_AND:
@@ -279,7 +272,7 @@ static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
                   /* Match */
                   if (or_type)
                     return 1;
-                   goto next_srt;
+		  goto next_srt;
                }
 
              /* Match failed */
@@ -417,11 +410,11 @@ static int compile_psrt (PScore_Regexp_Type *psrt, Score_Regexp_Type *srt,
      {
         unsigned int flags = psrt->flags;
 
-        if ((SCORE_GENERIC == (srt->header_type = psrt->header_type))
-	    && (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING))
+        if (SCORE_GENERIC == (srt->header_type = psrt->header_type))
           {
              *generic += 1;
-	     slrn_request_additional_header (psrt->keyword);
+	     slrn_request_additional_header (psrt->keyword,
+					     Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING);
           }
 	
         srt->not_flag = (0 != (flags & NOT_FLAG));
@@ -579,7 +572,7 @@ int slrn_open_score (char *group_name)
    Slrn_Apply_Score = 1;
 
 #ifndef SLRNPULL_CODE
-   if (generic && (Slrn_Prefer_Head || !Slrn_Server_Obj->sv_has_cmd("XHDR")))
+   if (generic && (Slrn_Prefer_Head || (1 != Slrn_Server_Obj->sv_has_cmd("XHDR Path"))))
      slrn_open_suspend_xover ();
 #endif
 
@@ -870,6 +863,9 @@ static int handle_include_line (char *file, char *line, Score_Context_Type *c)
 {
    char *dir, *f;
    char buf [SLRN_MAX_PATH_LEN];
+#ifdef __CYGWIN__
+   char convline [SLRN_MAX_PATH_LEN];
+#endif
    int status;
 
    line = slrn_skip_whitespace (line);
@@ -879,8 +875,13 @@ static int handle_include_line (char *file, char *line, Score_Context_Type *c)
    
    (void) slrn_trim_string (line);
 
-#if defined(IBMPC_SYSTEM)
+#ifdef __CYGWIN__
+   if (0 == slrn_cygwin_convert_path (line, convline, sizeof (convline)))
+     line = convline;
+#else
+# if defined(IBMPC_SYSTEM)
    slrn_os2_convert_path (line);
+# endif
 #endif
 
    /* Make a filename which is relative to dir of current file.
@@ -912,8 +913,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
     while (fgets (line, sizeof (line) - 1, fp))
      {
 	unsigned char *lp;
-	int not_flag;
-        
+	int ret, not_flag;
 	
 	(*linenum)++;
 
@@ -994,7 +994,6 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	       {
 		  if (!slrn_case_strncmp (lp, (unsigned char *) "Expires:", 8))
 		    {
-		       int ret;
 		       ret = has_score_expired (lp + 8, c->today);
 		       if (ret == -1)
 			 {
@@ -1042,35 +1041,36 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	  }
 	else not_flag = 0;
 
+	ret = -1;
 	psrt = (PScore_Regexp_Type *) slrn_safe_malloc (sizeof (PScore_Regexp_Type));
 	/* Otherwise the line is a kill one */
 	if (!slrn_case_strncmp (lp, (unsigned char *)"Subject:", 8))
-	  add_group_regexp (psrt, lp + 7, lp, SCORE_SUBJECT, not_flag);
+	  ret = add_group_regexp (psrt, lp + 7, lp, SCORE_SUBJECT, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"From:", 5))
-	  add_group_regexp (psrt, lp + 4, lp, SCORE_FROM, not_flag);
+	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_FROM, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Xref:", 5))
-	  add_group_regexp (psrt, lp + 4, lp, SCORE_XREF, not_flag);
+	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_XREF, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Newsgroup:", 10))
-	  add_group_regexp (psrt, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
+	  ret = add_group_regexp (psrt, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"References:", 11))
-	  add_group_regexp (psrt, lp + 10, lp, SCORE_REFERENCES, not_flag);
+	  ret = add_group_regexp (psrt, lp + 10, lp, SCORE_REFERENCES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Lines:", 6))
-	  add_group_regexp (psrt, lp + 5, lp, SCORE_LINES, not_flag);
+	  ret = add_group_regexp (psrt, lp + 5, lp, SCORE_LINES, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Date:", 5))
-	  add_group_regexp (psrt, lp + 4, lp, SCORE_DATE, not_flag);
+	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_DATE, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Age:", 4))
-	  add_group_regexp (psrt, lp + 3, lp, SCORE_AGE, not_flag);
+	  ret = add_group_regexp (psrt, lp + 3, lp, SCORE_AGE, not_flag);
 	else if (!slrn_case_strncmp (lp, (unsigned char *)"Message-Id:", 11))
-	  add_group_regexp (psrt, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
+	  ret = add_group_regexp (psrt, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
         else if (!slrn_case_strncmp (lp, (unsigned char *)"{:", 1))
 	  {
             if (lp[2] ==':')
 	      {
-		add_group_regexp (psrt, lp + 1, lp, SCORE_SUB_OR, not_flag);
+		ret = add_group_regexp (psrt, lp + 1, lp, SCORE_SUB_OR, not_flag);
 	      }
 	    else
 	      {
-		add_group_regexp (psrt, lp + 1, lp, SCORE_SUB_AND, not_flag);
+		ret = add_group_regexp (psrt, lp + 1, lp, SCORE_SUB_AND, not_flag);
 	      }
 	    if (phrase_score_file(file, fp, c, linenum, psrt))  return -1;
           }
@@ -1094,9 +1094,15 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		  return -1;
 	       }
 	     
-	     add_group_regexp (psrt, lpp, lp, SCORE_GENERIC, not_flag);
+	     ret = add_group_regexp (psrt, lpp, lp, SCORE_GENERIC, not_flag);
 	  }
-   
+	
+	if (ret == -1)
+	  {
+	     score_error ("No regular expression given.", line, *linenum, file);
+	     return -1;
+	  }
+	
 	if (sub_psrt == NULL)
 	  {
 	     if (c->pst->pregexp_list == NULL)
