@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -33,6 +33,9 @@
 # include <sys/types.h>
 # include <sys/stat.h>
 #endif
+#ifdef __MINGW32__
+# include <process.h>
+#endif
 
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
@@ -45,9 +48,9 @@
 #include "jdmacros.h"
 
 #include "slrn.h"
-#include "server.h"
-#include "misc.h"
 #include "util.h"
+#include "misc.h"
+#include "server.h"
 #include "startup.h"
 #include "snprintf.h"
 #if SLRN_USE_SLTCP 
