@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -59,6 +59,9 @@
 #  include <sys/types.h>
 #  include <sys/stat.h>
 # endif
+# ifdef __MINGW32__
+#  include <process.h>
+# endif
 #endif /* !VMS */
 
 #ifdef HAVE_SYS_WAIT_H
@@ -71,6 +74,7 @@
 
 #include <errno.h>
 
+#include "util.h"
 #include "server.h"
 #include "slrn.h"
 #include "group.h"
@@ -78,7 +82,6 @@
 #include "startup.h"
 #include "art.h"
 #include "score.h"
-#include "util.h"
 #include "snprintf.h"
 #include "chmap.h"
 #include "post.h"
@@ -1170,6 +1173,9 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
 					     (unsigned char *)"koi8"))
 	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("koi8-r");
+	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
+					     (unsigned char *)"ibm737"))
+	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-7");
 	  }
 	if (NULL == Slrn_Mime_Display_Charset)
 # endif
@@ -1225,11 +1231,17 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
    
    if (no_score_file == 0) read_score_file ();
    
-   hlp_file = getenv ("SLRNHELP");
-   if (hlp_file != NULL)
+   if (NULL == (hlp_file = getenv ("SLRNHELP")))
      {
-	slrn_parse_helpfile (hlp_file);
+	hlp_file = file;
+#ifdef VMS
+	slrn_snprintf (file, sizeof (file), "%s%s", Slrn_Lib_Dir, "help.txt");
+#else
+	slrn_snprintf (file, sizeof (file), "%s/%s", Slrn_Lib_Dir, "help.txt");
+#endif
      }
+	
+   slrn_parse_helpfile (hlp_file);
    
    if ((Slrn_Newsrc_File == NULL)
        && ((Slrn_Newsrc_File = slrn_map_file_to_host (Slrn_Server_Obj->sv_name)) == NULL))
@@ -1281,6 +1293,9 @@ If you want to create %s, add command line options:\r\n\
 	slrn_fclose (fp);
      }
    
+   /* make sure we have an entry for the server */
+   slrn_add_to_server_list (Slrn_Server_Obj->sv_name, NULL, NULL, NULL);
+   
    if (kill_logfile != NULL)
      {
 	Slrn_Kill_Log_FP = fopen(kill_logfile, "a");
@@ -1318,7 +1333,7 @@ If you want to create %s, add command line options:\r\n\
    if (Slrn_Server_Obj->sv_has_xover == 1)
      {
 	if (0 == (Slrn_Server_Obj->sv_has_xover = slrn_read_overview_fmt ()))
-	  slrn_message ("Broken OVERVIEW.FMT -- XOVER support disabled.");
+	  slrn_message ("OVERVIEW.FMT not RFC 2980 compliant -- XOVER support disabled.");
      }
    
    if (Slrn_Check_New_Groups || create_flag)
