@@ -30,6 +30,9 @@
 /* define if you want SSL support */
 #define SLRN_HAS_SSL_SUPPORT	0
 
+/* define if you want uudeview support */
+#define SLRN_HAS_UUDEVIEW	0
+
 /* define if you want slang interpreter support */
 #define SLRN_HAS_SLANG			1
 
@@ -112,7 +115,11 @@
 # endif
 #endif
 
-#if defined(__CYGWIN32__) || defined(__MINGW32__)
+#if defined(__CYGWIN32__) && !defined(__CYGWIN__)
+# define __CYGWIN__
+#endif
+
+#if defined(__CYGWIN__) || defined(__MINGW32__)
 # ifndef __WIN32__
 #  define __WIN32__
 # endif
@@ -163,7 +170,7 @@
 # define HAVE_FCNTL_H 1
 # define HAVE_LOCALE_H 1
 # define HAVE_SETLOCALE 1
-# if defined(__CYGWIN32__) || defined(__MINGW32__)
+# if defined(__CYGWIN__) || defined(__MINGW32__)
 #  define HAVE_DIRENT_H 1
 # else
 #  define HAVE_DIRECT_H 1
@@ -173,7 +180,7 @@
 #endif
 
 #if defined(__WIN32__)
-# if !defined(__CYGWIN32__) && !defined(__MINGW32__)
+# if !defined(__CYGWIN__) && !defined(__MINGW32__)
 #  define HAVE_WINSPOOL_H
 # endif
 #endif
@@ -196,7 +203,7 @@
 # define HAVE_ISPUNCT 1
 #endif
 
-#ifdef VMS
+#if defined(VMS) || defined(__os2__)
 # define USE_NOTEPAD_PRINT_CODE
 #endif
 
@@ -217,6 +224,9 @@
 #define HAVE_MEMCPY 1
 #define HAVE_MEMCHR 1
 
+/* undefine if your sprintf does not return an int */
+#define HAVE_ANSI_SPRINTF 1
+
 #define SLRN_SERVER_ID_NNTP 1
 #define SLRN_SERVER_ID_SPOOL 2
 
@@ -224,7 +234,7 @@
 #define SLRN_POST_ID_INEWS 2
 #define SLRN_POST_ID_PULL 3
 
-#if defined(IBMPC_SYSTEM)
+#if defined(IBMPC_SYSTEM) && !defined(__CYGWIN__)
 # define SLRN_PATH_SLASH_CHAR	'\\'
 #endif
 
