@@ -7,20 +7,14 @@
  * the feature will be turned off.
  */
 
-/* If non-zero, a User-Agent header will be generated instead of X-Newsreader. */
-#define SLRN_HAS_USER_AGENT	1
-
 /* If non-zero, a special message will be displayed when article is the last
  * one in a thread.
  */
 #define SLRN_HAS_END_OF_THREAD	1
 
-/* If you want to use uudeview's uulib instead of slrn's builtin one
- * change this to 1.  You will also need to edit the Makefile to point to
- * the uudeview include file and and library.  Only enable this if you
- * have uudeview installed on your system.
- */
-#define SLRN_HAS_UUDEVIEW	0
+/* Unset this if you have inews support enabled and do not want to allow
+ * users to set the inews command slrn invokes. */
+#define SLRN_HAS_USER_INEWS	1
 
 /* The sltcp interface is necessary for simultaneous tcp/ip connections,
  * but does not support most VMS systems. */
@@ -51,14 +45,21 @@
 # define SLRN_HAS_RNLOCK	1
 #endif
 
-#define SLRN_MAX_HEADER_FORMATS 10
+/* Number of messages slrnpull requests in a row to reduce the time it spends
+ * waiting for the server's response.  If your connection drops a lot and you
+ * are mainly downloading large messages, you might want to reduce this.
+ */
+#define SLRN_MAX_QUEUED	10
+
+#define SLRN_MAX_DISPLAY_FORMATS 10
 
 /* ----------------  LOCAL SPOOL and INEWS filenames and configuration ----------------------- */
 
 #if SLRN_HAS_INEWS_SUPPORT && ! defined(SLRN_INEWS_COMMAND)
   /* Note the -S flag.  slrn appends the signature to the file to be posted and
-   * the -S flag tells inews not to also do this.  The -h flag must be used.
-   */
+   * the -S flag tells inews not to also do this.  The -h flag must be used. */
+
+  /* Changing this here has no effect when using ./configure (on Unix) */
 # define SLRN_INEWS_COMMAND      "/usr/local/bin/inews -S -h"
 #endif
 
@@ -149,6 +150,7 @@
 #   define SLRN_ARTICLE_FILENAME	".article"
 #   define SLRN_FOLLOWUP_FILENAME	".followup"
 #   define SLRN_SIGNATURE_FILE		".signature"
+  /* Changing these here has no effect when using ./configure */
 #   ifndef SLRN_SENDMAIL_COMMAND
 #    define SLRN_SENDMAIL_COMMAND	"/usr/lib/sendmail -oi -t -oem -odb"
 #   endif
@@ -184,6 +186,9 @@
 /* slrnpull outgoing post directory. */
 #define SLRNPULL_OUTGOING_DIR	"out.going"
 
+/* slrnpull failed outgoing post directory. */
+#define SLRNPULL_OUTGOING_BAD_DIR	"rejects"
+
 /* If set to 1, slrn will write files to the outgoing directory with group
  * write permission.  This assumes that the outgoing directory has been 
  * created with the setgid bit enabled, which slrnpull will do if this is
