@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 
@@ -804,7 +804,7 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
 	if (server_max >= server_min)
 	  log_message ("The server may contain as many as %d articles in this group",
 		       server_max - server_min + 1);
-	log_message ("Only examining last %u articles since this is a new group", max);
+	log_message ("Only examining last %u articles since this is a new group", g->max_to_get);
      }
 
    status = nntp_server_vcmd (s, "XHDR Message-Id %u-", max);
@@ -973,7 +973,7 @@ static int write_xover_line (FILE *fp, Slrn_XOver_Type *xov) /*{{{*/
    if ((EOF == fprintf (fp,
 			"%d\t%s\t%s\t%s\t%s\t%s\t%d\t%d",
 			xov->id, xov->subject_malloced, 
-			xov->from, xov->date, xov->message_id,
+			xov->from_malloced, xov->date, xov->message_id,
 			xov->references, xov->bytes, xov->lines))
        || ((xov->xref != NULL) && (xov->xref[0] != 0)
 	   && (EOF == fprintf (fp, "\tXref: %s", xov->xref)))
@@ -1291,6 +1291,8 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 	slrn_free (bodies[i]);
 	slrn_free (heads[i]);
 	slrn_free (xovs[i].subject_malloced);
+	slrn_free (xovs[i].from_malloced);
+	slrn_free_additional_headers (xovs[i].add_hdrs);
      }
    return ret;
 }
@@ -1377,6 +1379,7 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
    g->max = g->server_max;
    
    (void) slrn_close_score ();
+   slrn_clear_requested_headers ();
    slrn_free ((char *) numbers);
    return 0;
 }
@@ -1766,17 +1769,18 @@ static void usage (char *pgm) /*{{{*/
 {
    log_error ("%s usage: %s [options]\n\
  Options:\n\
-  -h HOSTNAME          name of NNTP server\n\
-  -d SPOOLDIR          Spool Directory associated with server\n\
-  --expire             Perform expiration but do not pull news\n\
-  --post-only          Post news but do not pull new news\n\
-  --no-post            Do not post news\n\
-  --new-groups         Get new groups\n\
-  --kill-score SCORE   Kill articles with a score below SCORE\n\
-  --kill-log FILE      Keep a log of all killed articles in FILE\n\
-  --logfile FILE       Use FILE as the log file\n\
-  --debug FILE         Write dialogue with server to FILE\n\
-  --version            Show version number\n\
+  -d SPOOLDIR          Spool directory to use.\n\
+  -h HOSTNAME          Hostname of NNTP server to connect to.\n\
+  --debug FILE         Write dialogue with server to FILE.\n\
+  --expire             Perform expiration, but do not pull news.\n\
+  --help               Print this usage information.\n\
+  --kill-log FILE      Keep a log of all killed articles in FILE.\n\
+  --kill-score SCORE   Kill articles with a score below SCORE.\n\
+  --logfile FILE       Use FILE as the log file.\n\
+  --new-groups         Get a list of new groups.\n\
+  --no-post            Do not post news.\n\
+  --post-only          Post news, but do not pull new news.\n\
+  --version            Show the version number.\n\
 ",
 	      pgm, pgm);
    close_log_files ();
@@ -2057,39 +2061,26 @@ static int get_new_groups (NNTP_Type *s)
 	*p = 0;
 	
 	parse_error = 0;
-	i = 0;
-	
-	do
-	  {
-	     switch (nntp_server_cmd (s, line))
-	       {
-		case OK_NEWGROUPS:
-		  break;
-		  
-		case ERR_FAULT:
-		  return 0;
-		  
-		case ERR_COMMAND:
-		  log_message ("Server does not implement NEWGROUPS command.");
-		  return 0;
 
-		case ERR_NOAUTH:
-		  if ((i++ != 0) || (-1 == nntp_authorization (s)))
-		    {
-		       slrn_exit_error ("Authorization failed.");
-		       return -1;
-		    }
-		  break;
-		  
-		default:
-		  slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
-				s->rspbuf);
-		  
-		  goto parse_error_label;
-	       }
+	switch (nntp_server_cmd (s, line))
+	  {
+	   case OK_NEWGROUPS:
+	     break;
+	     
+	   case ERR_FAULT:
+	     return 0;
+	     
+	   case ERR_COMMAND:
+	     log_message ("Server does not implement NEWGROUPS command.");
+	     return 0;
+	     
+	   default:
+	     slrn_message ("Server failed to return proper response to NEWGROUPS:\n%s\n",
+			   s->rspbuf);
+	     
+	     goto parse_error_label;
 	  }
-	while (i++ == 1); /* loops (at most) once */
-      
+	
 	num = 0;
 	while (1 == nntp_read_line (s, line, sizeof (line)))
 	  {
@@ -2381,12 +2372,14 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	  {
 	     slrn_free (header);
 	     slrn_free (xov.subject_malloced);
+	     slrn_free (xov.from_malloced);
 	     fclose (xov_fp);
 	     return -1;
 	  }
 	
 	slrn_free (header);
 	slrn_free (xov.subject_malloced);
+	slrn_free (xov.from_malloced);
      }
    
    return slrn_fclose (xov_fp);
@@ -2592,9 +2585,9 @@ static int read_authinfo (void)
    slrn_trim_string (pass);
    slrn_trim_string (name);
    
-   if ((0 == strlen (pass)) || (0 == strlen (name)))
+   if (0 == strlen (name))
      {
-	log_error ("Invalid name or password.");
+	log_error ("Empty user name.  Please check syntax of the authinfo file.");
 	return -1;
      }
    
