@@ -15,8 +15,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -27,7 +27,10 @@
 #include <string.h>
 
 #include <stdlib.h>
-#include <unistd.h>
+
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 
 #include <errno.h>
 #include <ctype.h>
@@ -50,7 +53,7 @@
 # include <winsock.h>
 # define USE_WINSOCK_SLTCP	1
 #else
-# if defined(__MINGW32__)
+# if defined(__WIN32__)
 #  define Win32_Winsock
 #  include <windows.h>
 #  define USE_WINSOCK_SLTCP	1
@@ -110,6 +113,8 @@ extern int h_errno;
 #endif
 
 #include "sltcp.h"
+#include "snprintf.h"
+#include "util.h"
 
 struct _SLTCP_Type
 {
@@ -144,7 +149,7 @@ struct _SLTCP_Type
 #endif
 
 int (*SLTCP_Interrupt_Hook) (void);
-int SLtcp_TimeOut_Secs = 120;
+int SLtcp_TimeOut_Secs = SLRN_SLTCP_TIMEOUT_SECS;
 
 static int TCP_Verbose_Reporting = 0;
 
@@ -190,7 +195,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 	     max_retries--;
 	     if (max_retries && (h_errno == TRY_AGAIN))
 	       {
-		  sleep (1);
+		  slrn_sleep (1);
 		  continue;
 	       }
 #endif
@@ -543,14 +548,14 @@ static int check_errno (void)
 #ifdef EAGAIN
    if (errno == EAGAIN) 
      {
-	sleep (1);
+	slrn_sleep (1);
 	return 0;
      }
 #endif
 #ifdef EWOULDBLOCK
    if (errno == EWOULDBLOCK)
      {
-	sleep (1);
+	slrn_sleep (1);
 	return 0;
      }
 #endif
@@ -941,7 +946,7 @@ int sltcp_vfprintf (SLTCP_Type *tcp, char *fmt, va_list ap) /*{{{*/
    if ((tcp == NULL) || (-1 == tcp->tcp_fd))
      return -1;
    
-   vsprintf (buf, fmt, ap);
+   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
    
    return sltcp_fputs (tcp, buf);
 }
