@@ -2,6 +2,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -14,24 +15,23 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
-#include "slrnfeat.h"
+#ifndef SLRNPULL_CODE
+# include "slrnfeat.h"
+#endif
 #include <stdio.h>
 #include <string.h>
-#include <ctype.h>
+#include <time.h>
 
-#include "slrn.h"
-#include "util.h"
+#ifndef SLRNPULL_CODE
+# include "slrn.h"
+# include "util.h"
+#endif
 
-#ifndef isdigit
-# define isdigit(x) (((x)>='0')&&((x)<='9'))
-#endif
-#ifndef isalpha
-# define isalpha(x) ((((x)>='A')&&((x)<='Z')) || (((x)>='A')&&((x)<='Z')))
-#endif
+#include "jdmacros.h"
 
 /* This routine parses the date field of a header and returns it as a time_t
  * object.  The main difficulity is the lack of uniformity in the date formats
@@ -66,7 +66,7 @@
  * Beware: the calculations involving leap years, etc... are naive.
  */
 
-#define THE_YEAR_0		1992L  /* leap year */
+#define THE_YEAR_0		1970L  /* no leap year */
 #define IS_LEAP_YEAR(y)		(((y) % 4) == 0)
 
 typedef struct 
@@ -78,19 +78,21 @@ Timezone_Table_Type;
 
 static Timezone_Table_Type Timezone_Table [] =
 {
-   {"EDT", -500},
-   {"PST", -800},
-   {"CST", -600},
-   {"MST", -700},
+   {"EST", -500},		/* US Eastern Standard Time*/
+   {"CST", -600},		/* US Central */
+   {"MST", -700},		/* US Mountain */
+   {"PST", -800},		/* US Pacific  */
+   {"EDT", -400},		/* US Eastern Daylight Time */
+   {"CDT", -500},		/* US Central Daylight Time */
+   {"MDT", -600},		/* US Mountain Daylight Time */
+   {"PDT", -700},		/* US Pacific Daylight Time */
+   {"CET", 100},		       /* Central European */
    {"MET", 100},		       /* Middle European */
    {"MEZ", 100},		       /* Middle European */
    {"MSK", 300},		       /* Moscow */
-   {"CET", 100},		       /* Central European */
-   {"HKT", 800},		       /* Central European */
-   {"JST", 900},		       /* Central European */
-#if 1
+   {"HKT", 800},
+   {"JST", 900},
    {"KST", 900},		       /* Korean Standard */
-#endif
    {"CAST", 930},		       /* Central Autsralian */
    {"EAST", 1000},		       /* Eastern Autsralian */
    {"NZST", 1200},		       /* New Zealand Autsralian */
@@ -283,7 +285,8 @@ long slrn_date_to_order_parm (char *date)
    if (isalpha (*date))
      {
 	sign = 1;
-	if ((*date != 'G') && (*date != 'M') && (*date != 'T'))
+	if (((*date != 'G') || (*(date+1) != 'M') || (*(date+2) != 'T')) &&
+	    ((*date != 'U') || (*(date+1) != 'T')))
 	  tz = parse_timezone (date);
      }
 
@@ -298,7 +301,7 @@ long slrn_date_to_order_parm (char *date)
    /* add that to number of days since beginning of time */
    year -= THE_YEAR_0;
    day += year * 365 + year / 4;
-   if ((year % 4) == 1) day++;
+   if ((year % 4) == 3) day++;
    
    /* Adjust hours for timezone */
    hours -= tz / 100;
@@ -310,3 +313,25 @@ long slrn_date_to_order_parm (char *date)
    
    return seconds;
 }
+
+/* Formats a given "date" (as understood by slrn_date_to_order_parm) according
+ * to the format specification "format" (see strftime(3)) */
+void slrn_strftime (char *s, size_t max, const char *format, char *date,
+		    int use_localtime)
+{
+   time_t date_t;
+   struct tm *date_tm;
+   
+   date_t = (time_t) slrn_date_to_order_parm (date);
+   
+#if defined(VMS) || defined(__BEOS__)
+   (void) use_localtime; /* gmtime is broken on BEOS */
+#else
+   if (0 == use_localtime)
+     date_tm = gmtime (&date_t);
+   else
+#endif
+     date_tm = localtime (&date_t);
+   
+   (void) strftime (s, max, format, date_tm);
+}
