@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 /* Read startup .slrnrc file */
 
@@ -251,6 +251,7 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
    char *key = table->string_args[3];
    SLKeyMap_List_Type *kmap = NULL;
    char **obsolete_names = NULL;
+   int failure;
    
    (void) argc;
 
@@ -267,7 +268,18 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
    else if (!strcmp (map, "readline")) kmap = Slrn_RLine_Keymap;
    else slrn_exit_error ("%s: line %d:\n%sNo such keymap: %s", This_File, This_Line_Num, This_Line, map);
    
-   if (SLang_define_key (key, fun, kmap) != 0)
+   if ((kmap == Slrn_Keymap_RLI->keymap) &&
+       (NULL == SLang_find_key_function(fun, kmap)))
+     {
+	SLKeymap_Function_Type *tmp = kmap->functions;
+	kmap->functions = Slrn_Custom_Readline_Functions;
+	failure = SLang_define_key (key, fun, kmap);
+	kmap->functions = tmp;
+     }
+   else
+     failure = SLang_define_key (key, fun, kmap);
+   
+   if (failure)
      {
 	slrn_exit_error ("%s: line %d:\n%serror defining key.", This_File, This_Line_Num, This_Line);
      }
@@ -470,6 +482,8 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 
      {"use_flow_control",	&Slrn_Use_Flow_Control},
      {"abort_unmodified_edits", &Slrn_Abort_Unmodified},
+     {"editor_uses_mime_charset", &Slrn_Editor_Uses_Mime_Charset},
+     {"mail_editor_is_mua",	&Slrn_Mail_Editor_Is_Mua},
      {"auto_mark_article_as_read", &Slrn_Del_Article_Upon_Read},
      {"simulate_graphic_chars", &Slrn_Simulate_Graphic_Chars},
 #if 0 /* This does not work yet */
@@ -481,6 +495,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"ignore_signature", &Slrn_Sig_Is_End_Of_Article},
      {"reject_long_lines", &Slrn_Reject_Long_Lines},
      {"netiquette_warnings", &Slrn_Netiquette_Warnings},
+     {"generate_date_header", &Slrn_Generate_Date_Header},
      {"generate_message_id", &Slrn_Generate_Message_Id},
 #if SLRN_HAS_STRICT_FROM
      {"generate_email_from", NULL},
@@ -490,8 +505,10 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"display_cursor_bar", &Slrn_Display_Cursor_Bar},
 #if SLRN_HAS_NNTP_SUPPORT
      {"broken_xref", &Slrn_Broken_Xref},
+     {"force_authentication", &Slrn_Force_Authentication},
 #else
      {"broken_xref", NULL},
+     {"force_authentication", NULL},
 #endif
      {"color_by_score", &Slrn_Color_By_Score},
      {"highlight_unread_subjects", &Slrn_Highlight_Unread},
@@ -516,6 +533,11 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"prefer_head", &Slrn_Prefer_Head},
 #if SLRN_HAS_MIME
      {"use_metamail", &Slrn_Use_Meta_Mail},
+#endif
+#if SLRN_HAS_UUDEVIEW
+     {"use_uudeview", &Slrn_Use_Uudeview},
+#else
+     {"use_uudeview", NULL},
 #endif
      {"lines_per_update", &Slrn_Reads_Per_Update},
      {"min_high_score", &Slrn_High_Score_Min},
@@ -541,8 +563,10 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 #endif
 #if SLRN_HAS_MIME
      {"use_mime", &Slrn_Use_Mime},
+     {"fold_headers", &Slrn_Fold_Headers},
 #else
      {"use_mime", NULL},
+     {"fold_headers", NULL},
 #endif
 #if SLRN_HAS_GROUPLENS
      {"use_grouplens", &Slrn_Use_Group_Lens},
@@ -1268,7 +1292,7 @@ static Server_List_Type *find_server (char *host) /*{{{*/
 
 /*}}}*/
 
-static int add_to_server_list (char *host, char *file, char *username, char *password) /*{{{*/
+int slrn_add_to_server_list (char *host, char *file, char *username, char *password) /*{{{*/
 {
    Server_List_Type *s;
    
@@ -1312,7 +1336,7 @@ static int server_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
       
    (void) argc;
    
-   return add_to_server_list (the_host, the_file, NULL, NULL);
+   return slrn_add_to_server_list (the_host, the_file, NULL, NULL);
 }
 
 /*}}}*/
@@ -1354,7 +1378,7 @@ static int nnrp_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
    char *pass = table->string_args[3];
 
    (void) argc;
-   return add_to_server_list (server, NULL, name, pass);
+   return slrn_add_to_server_list (server, NULL, name, pass);
 #else
    (void) argc;
    (void) table;
@@ -1377,10 +1401,7 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
    if (s == NULL)
      return 0;
    
-   if ((s->username == NULL) || (s->password == NULL))
-     return 0;
-   
-   if (*s->username == 0)
+   if ((s->username == NULL) || (*s->username == 0))
      {
 	*buf = 0;
 	if (do_newline)
@@ -1390,13 +1411,13 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
 	     do_newline = 0;
 	  }
 
-	if (-1 == slrn_read_input ("Username", NULL, buf, 1, 0))
+	if (-1 == slrn_read_input ("Username: ", NULL, buf, 1, 0))
 	  return -1;
-	SLFREE (s->username);
+	slrn_free (s->username);
 	s->username = slrn_safe_strmalloc (buf);
      }
 
-   if (*s->password == 0)
+   if ((s->password == NULL) || (*s->password == 0))
      {
 	*buf = 0;
 	if (do_newline)
@@ -1405,9 +1426,9 @@ int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
 	       putc ('\n', stdout);
 	  }
 
-	if (-1 == slrn_read_input_no_echo ("Password", NULL, buf, 1, 0))
+	if (-1 == slrn_read_input_no_echo ("Password: ", NULL, buf, 1, 0))
 	  return -1;
-	SLFREE (s->password);
+	slrn_free (s->password);
 	s->password = slrn_safe_strmalloc (buf);
      }
 
@@ -1550,11 +1571,11 @@ int slrn_read_startup_file (char *name) /*{{{*/
 
 static int include_file_fun (int argc, SLcmd_Cmd_Table_Type *table)
 {
-   char *file;
+   char file [SLRN_MAX_PATH_LEN];
    
    (void) argc;
    
-   file = table->string_args [1];
+   slrn_make_home_filename (table->string_args [1], file, sizeof (file));
 
    if (-1 == slrn_read_startup_file (file))
      slrn_exit_error ("%s: line %d:\n%sFile not opened: %s",
