@@ -14,8 +14,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #ifndef _SLRN_UTIL_H
 #define _SLRN_UTIL_H
@@ -39,6 +39,10 @@ extern int slrn_is_absolute_path (char *);
 extern void slrn_os2_convert_path (char *);
 extern void slrn_os2_make_fat (char *, size_t, char *, char *);
 #endif
+#ifdef __CYGWIN__
+extern int slrn_cygwin_convert_path (char *, char *, size_t);
+#endif
+extern unsigned int slrn_sleep (unsigned int);
 
 extern char *slrn_simple_strtok (char *, char *);
 extern char *slrn_strchr (char *, char);
@@ -56,6 +60,7 @@ extern char *slrn_strnmalloc (char *, unsigned int, int);
 extern char *slrn_malloc (unsigned int, int, int);
 extern char *slrn_realloc (char *, unsigned int, int);
 extern void slrn_free (char *);
+extern void slrn_free_argc_argv_list (unsigned int, char **);
 
 extern char *slrn_fix_regexp (char *);
 
@@ -65,4 +70,10 @@ extern char *slrn_fix_regexp (char *);
 extern void slrn_exit_error (char *, ...);
 char *slrn_make_from_string (void);
 
+typedef struct Slrn_Group_Range_Type
+{
+   char *name;
+   int min, max;
+} Slrn_Group_Range_Type;
+
 #endif				       /* _SLRN_UTIL_H */
