@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 #include "slrnfeat.h"
@@ -734,6 +734,7 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 			       int no_slop) /*{{{*/
 {
    FILE *outfp = NULL;
+   int is_mbox = -1;
    char buf[MAX_ARTICLE_LINE_LEN];
    
    while (NULL != fgets (buf, sizeof (buf), fp))
@@ -741,6 +742,18 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 	char *file;
 	int mode = 0;
 	unsigned int buflen;
+	
+	if (is_mbox == -1)
+	  {
+	     if ((!strncmp (buf, "From", 4)) &&
+		 ((unsigned char)buf[4] <= ' '))
+	       {
+		  is_mbox = 1;
+		  continue;
+	       }
+	     else
+	       is_mbox = 0;
+	  }
 
 	/* We are looking for 'begin ddd filename' */
 	if (strncmp (buf, "begin ", 6))
@@ -769,6 +782,22 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 	     unsigned int len, write_len;
 	     unsigned char out[60], *outp, *outmax, *b, *bmax;
 	     
+	     if ((is_mbox) && (!strncmp (buf, "From", 4)) &&
+		 ((unsigned char)buf[4] <= ' '))
+	       {
+		  do
+		    {
+		       if (NULL == fgets_with_counter (buf, sizeof (buf) - 1,
+						       fp, &buflen))
+			 {
+			    slrn_error ("Unexpected end of file.\n");
+			    return -1;
+			 }
+		    }
+		  while (buflen);
+		  continue;
+	       }
+	     
 	     if (((*buf & 0x3F) == ' ') && ((*buf + 1) == '\0'))
 	       {
 		  /* we may be on the last line before the end.  Lets check */
