@@ -15,13 +15,14 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define VERSION "0.9.7.0" /* Make sure to update version info in VMSMAKE.COM!!! */
-#undef PATCH_LEVEL
-#define SLRN_RELEASE_DATE "2001-03-28"
+#define VERSION "0.9.7.2" /* Make sure to update version info in VMSMAKE.COM!!! */
+#define SLRN_VERSION_NUMBER 90702
+/*#define PATCH_LEVEL ""*/
+#define SLRN_RELEASE_DATE "2001-08-20"
 
 #ifdef PATCH_LEVEL
 # define SLRN_VERSION (VERSION "pl" PATCH_LEVEL)
@@ -31,6 +32,8 @@
 
 #ifndef SLRNPULL_CODE
 extern char *Slrn_Version;
+extern int Slrn_Version_Number;
+extern char *Slrn_Date;
 extern char *slrn_get_os_name (void);
 extern void slrn_show_version (void);
 #endif
