@@ -16,8 +16,8 @@
  more details.
 
  You should have received a copy of the GNU General Public License along
- with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ with this program; if not, write to the Free Software Foundation, Inc.,
+ 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 #include "config.h"
 
@@ -42,9 +42,7 @@
 #ifndef SLRNPULL_CODE
 # include "group.h"
 # include "art.h"
-# if SLRN_HAS_MIME
-#  include "mime.h"
-# endif
+# include "mime.h"
 #endif /* NOT SLRNPULL_CODE */
 
 #include "xover.h"
@@ -53,13 +51,14 @@
 # include "server.h"
 
 static int extract_id_from_xref (char *);
-static Slrn_Header_Line_Type *copy_add_headers (Slrn_Header_Line_Type *);
+static void rearrange_add_headers (void);
 static int Suspend_XOver_For_Kill = 0;
 #endif
 
-static Slrn_Header_Line_Type *Additional_Headers = NULL;
+static Slrn_Header_Line_Type *copy_add_headers (Slrn_Header_Line_Type*, char);
 static Slrn_Header_Line_Type *Unretrieved_Headers = NULL;
 static Slrn_Header_Line_Type *Current_Add_Header = NULL;
+static Slrn_Header_Line_Type *Additional_Headers = NULL;
 #if SLRN_HAS_FAKE_REFS
 static char *In_Reply_To;
 #endif
@@ -76,20 +75,18 @@ static Slrn_Header_Line_Type Xover_Headers [] = /*{{{*/
      {"Message-ID", 10, NULL, NULL},
 #define REFS_OFFSET     4
      {"References", 10, NULL, NULL},
-#define LINES_OFFSET    5
-     {"Lines", 5, NULL, NULL},
-#define XREF_OFFSET     6
-     {"Xref", 4, NULL, NULL},
    /* Technically, bytes is in the overview file.  However, this value
-    * is not stored in slrn's article structure.
-    */
-#define BYTES_OFFSET    7
-     {"Bytes", 5, NULL, NULL}
-#define XOVER_HEADERS_MAX 8
+    * is not stored in slrn's article structure. */
+#define BYTES_OFFSET    5
+     {"Bytes", 5, NULL, NULL},
+#define LINES_OFFSET    6
+     {"Lines", 5, NULL, NULL},
 };
 
 /*}}}*/
 
+static char *Xref;
+
 /* The pointers in the above structure will point to the following buffer. */
 static char *Malloced_Headers;
 
@@ -101,11 +98,12 @@ static void parse_headers (void) /*{{{*/
    
    /* reset all pointers */
    
-   for (i = 0; i < XOVER_HEADERS_MAX; i++)
+   for (i = 0; i < 7; i++)
      Xover_Headers[i].value = NULL;
+   Xref = NULL;
    
    for (addh = Additional_Headers; addh != NULL; addh = addh->next)
-     addh->value = NULL;
+     addh->value = "";
    
 #if SLRN_HAS_FAKE_REFS
    In_Reply_To = NULL;
@@ -130,26 +128,31 @@ static void parse_headers (void) /*{{{*/
 	colon++;
 	if (*colon == ' ') colon++;  /* space is required to be there */
 	
-	for (i = 0; i < XOVER_HEADERS_MAX; i++)
+	for (i = 0; i < 7; i++)
 	  {
-	     if ((Xover_Headers[i].value != NULL)
-		 || (len != Xover_Headers[i].name_len)
-		 || (slrn_case_strcmp ((unsigned char *)h, 
-				       (unsigned char *) Xover_Headers[i].name)))
-	       continue;
-	     
-	     Xover_Headers[i].value = colon;
-	     break;
+	     if ((Xover_Headers[i].value == NULL)
+		 && (len == Xover_Headers[i].name_len)
+		 && (!slrn_case_strcmp ((unsigned char *)h, 
+					(unsigned char *) Xover_Headers[i].name)))
+	       {
+		  Xover_Headers[i].value = colon;
+		  break;
+	       }
+	     else if ((Xref == NULL) && (len == 4) &&
+		      (!slrn_case_strcmp ((unsigned char *)h, (unsigned char *)"Xref")))
+	       {
+		  Xref = colon;
+		  break;
+	       }
 	  }
 	
 	for (addh = Additional_Headers; addh != NULL; addh = addh->next)
-	  if ((addh->value == NULL)
-	      && (len == addh->name_len)
+	  if ((len == addh->name_len)
 	      && (0 == slrn_case_strcmp ((unsigned char *)h,
 					 (unsigned char *) addh->name)))
 	    {
 #if SLRN_HAS_MIME
-	       if (Slrn_Use_Mime)
+	       if (Slrn_Use_Mime & MIME_DISPLAY)
 		 slrn_rfc1522_decode_string (colon);
 #endif
 	       addh->value = colon;
@@ -269,25 +272,28 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
        )
      refs = "";
 
-   if (NULL == (xref = Xover_Headers[XREF_OFFSET].value))
+   if (NULL == (xref = Xref))
      xref = "";
    if (NULL == (lines = Xover_Headers[LINES_OFFSET].value))
      lines = "";
    if (NULL == (bytes = Xover_Headers[BYTES_OFFSET].value))
      bytes = "";
    
-   len = strlen (subj) + strlen (from) + strlen (date) + strlen (msgid)
-     + strlen (refs) + strlen (xref) + 6;
-
+   xov->subject_malloced = slrn_strmalloc (subj, 0);
+   if (xov->subject_malloced == NULL)
+     return -1;
+   
+   len = strlen (from) + strlen (date) + strlen (msgid) + strlen (refs) +
+     strlen (xref) + 5;
+   
    buf = slrn_malloc (len, 0, 1);
    if (buf == NULL)
-     return -1;
-
-   xov->subject_malloced = buf;
-   strcpy (buf, subj); /* safe */
-   buf += strlen (subj) + 1;
+     {
+	slrn_free (xov->subject_malloced);
+	return -1;
+     }
    
-   xov->from = buf;
+   xov->from_malloced = buf;
    strcpy (buf, from); /* safe */
    buf += strlen (from) + 1;
 
@@ -313,12 +319,9 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 #ifndef SLRNPULL_CODE
    if (id == -1)
      id = extract_id_from_xref (xov->xref);
-   
-   if (!Slrn_Server_Obj->sv_has_xover || Suspend_XOver_For_Kill)
-     xov->add_hdrs = copy_add_headers (Additional_Headers);
-   else
 #endif
-     xov->add_hdrs = NULL;
+   
+   xov->add_hdrs = copy_add_headers (Additional_Headers, 0);
 
    xov->id = id;
 
@@ -327,23 +330,22 @@ static int parsed_headers_to_xover (int id, Slrn_XOver_Type *xov) /*{{{*/
 
 /*}}}*/
 
-char *slrn_get_extra_xover_header (char *hdr) /*{{{*/
+char *slrn_extract_add_header (Slrn_Header_Type *h, char *hdr) /*{{{*/
 {
-   Slrn_Header_Line_Type *h;
    unsigned int len;
+   Slrn_Header_Line_Type *l;
    
-   h = Additional_Headers;
    len = strlen (hdr);
-   
-   while (h != NULL)
+   if ((len > 2) && (hdr[len-1] == ' ') && (hdr[len-2] == ':'))
+     len -= 2;
+   for (l = h->add_hdrs; l != NULL; l = l->next)
      {
-	if ((len == h->name_len)
-	    && (0 == slrn_case_strcmp ((unsigned char *)h->name,
-				       (unsigned char *)hdr)))
-	  return h->value;
-	
-	h = h->next;
+	if ((l->name_len == len) &&
+	    (0 == slrn_case_strncmp ((unsigned char *) l->name,
+				     (unsigned char *) hdr, len)))
+	  return l->value;
      }
+   
    return NULL;
 }
 
@@ -370,14 +372,14 @@ void slrn_map_xover_to_header (Slrn_XOver_Type *xov, Slrn_Header_Type *h)
    
    h->subject = xov->subject_malloced;
    h->number = xov->id;
-   h->from = xov->from;
+   h->from = xov->from_malloced;
    h->date = xov->date;
    h->refs = xov->references;
    h->lines = xov->lines;
    h->xref = xov->xref;
    h->add_hdrs = xov->add_hdrs;
    
-   /* Since the subject has been malloced, the message_id pointer can be changed.
+   /* Since the strings have been malloced, the message_id pointer can be changed.
     */
    m = xov->message_id;
    while ((*m != '<') && (*m != 0))
@@ -395,6 +397,94 @@ void slrn_map_xover_to_header (Slrn_XOver_Type *xov, Slrn_Header_Type *h)
 				(unsigned char *) h->msgid + strlen (h->msgid));
 }
 
+typedef struct Overview_Fmt_Type
+{
+   char *name;
+   char full;
+   char **value;
+   struct Overview_Fmt_Type *next;
+}
+Overview_Fmt_Type;
+
+Overview_Fmt_Type *Overview_Fmt;
+
+/* returns zero if OVERVIEW.FMT does not contain the standard headers */
+int slrn_read_overview_fmt (void) /*{{{*/
+{
+#ifdef SLRNPULL_CODE
+   return 1;
+#else
+   char line[NNTP_BUFFER_SIZE];
+   Overview_Fmt_Type *tmp;
+   
+   while (Overview_Fmt != NULL)
+     {
+	tmp = Overview_Fmt->next;
+	slrn_free (Overview_Fmt->name);
+	slrn_free ((char *) Overview_Fmt);
+	Overview_Fmt = tmp;
+     }
+   
+   if (OK_GROUPS == Slrn_Server_Obj->sv_list ("OVERVIEW.FMT"))
+     {
+	int len, i = 0;
+	
+	while ((i < 7) && (Slrn_Server_Obj->sv_read_line (line, sizeof (line)) == 1))
+	  {
+	     if ((len = strlen (line)) && (line[len-1] == ':'))
+	       line[len-1] = 0;
+	     if (slrn_case_strcmp ((unsigned char*) line,
+				   (unsigned char*) Xover_Headers[i++].name))
+	       {
+		  while (Slrn_Server_Obj->sv_read_line (line, sizeof (line)) == 1)
+		    continue; /* discard remaining output */
+		  return 0;
+	       }
+	  }
+	if (i == 0)
+	  /* This is probably due to a broken firewall configuration;
+	   * assume XOVER is working anyway. */
+	  return 1;
+	else if (i != 7)
+	  return 0;
+	
+	tmp = NULL;
+	while (Slrn_Server_Obj->sv_read_line (line, sizeof (line)) == 1)
+	  {
+	     char *p;
+	     Overview_Fmt_Type* new_entry;
+	     
+	     if (NULL == (p = strchr (line, ':')))
+	       p = "";
+	     else
+	       *p++ = 0;
+	     
+	     if (NULL == (new_entry = (Overview_Fmt_Type*)
+			  slrn_malloc (sizeof (Overview_Fmt_Type), 1, 0)))
+	       break;
+	     if (NULL == (new_entry->name = slrn_strmalloc (line, 0)))
+	       {
+		  slrn_free ((char*)new_entry);
+		  break;
+	       }
+	     
+	     if (!slrn_case_strcmp ((unsigned char*) line, (unsigned char*)"Xref"))
+	       new_entry->value = &Xref;
+	     if (!strcmp (p, "full"))
+	       new_entry->full = 1;
+	     
+	     if (tmp == NULL)
+	       Overview_Fmt = new_entry;
+	     else
+	       tmp->next = new_entry;
+	     tmp = new_entry;
+	  }
+     }
+   return 1;
+#endif /* NOT SLRNPULL_CODE */
+}
+/*}}}*/
+
 void slrn_free_additional_headers (Slrn_Header_Line_Type *h) /*{{{*/
 {
    Slrn_Header_Line_Type *tmp;
@@ -411,17 +501,26 @@ void slrn_free_additional_headers (Slrn_Header_Line_Type *h) /*{{{*/
 
 void slrn_clear_requested_headers (void) /*{{{*/
 {
+   Overview_Fmt_Type *o;
+   
    slrn_free_additional_headers (Additional_Headers);
    Additional_Headers = NULL;
    Unretrieved_Headers = NULL;
    Current_Add_Header = NULL;
+   
+   for (o = Overview_Fmt; o != NULL; o = o->next)
+     {
+	if (o->value != &Xref)
+	  o->value = NULL;
+     }
 }
 
 /*}}}*/
 
-void slrn_request_additional_header (char *hdr) /*{{{*/
+void slrn_request_additional_header (char *hdr, int expensive) /*{{{*/
 {
    Slrn_Header_Line_Type *h, **hh;
+   Overview_Fmt_Type *o;
    unsigned int len;
    
    h = Additional_Headers;
@@ -442,12 +541,24 @@ void slrn_request_additional_header (char *hdr) /*{{{*/
 		slrn_malloc (sizeof (Slrn_Header_Line_Type), 0, 0)))
      return;
    
-   if (NULL == (h->name = slrn_strmalloc (hdr, 0)))
+   for (o = Overview_Fmt; o != NULL; o = o->next)
+     {
+	if ((o->value == NULL) &&
+	    !slrn_case_strcmp ((unsigned char*)hdr,
+			       (unsigned char*)o->name))
+	  {
+	     o->value = &h->value;
+	     break;
+	  }
+     }
+   
+   if (((o == NULL) && (expensive == 0))
+       || (NULL == (h->name = slrn_strmalloc (hdr, 0))))
      {
 	slrn_free ((char *)h);
 	return;
      }
-
+   
    h->name_len = len;
    h->value = NULL;
    h->next = NULL;
@@ -462,7 +573,60 @@ void slrn_request_additional_header (char *hdr) /*{{{*/
 
 /*}}}*/
 
+static Slrn_Header_Line_Type *copy_add_headers (Slrn_Header_Line_Type *l, /*{{{*/
+						char all)
+{
+   Slrn_Header_Line_Type *retval = NULL, *old = NULL, *copy;
+   unsigned int len;
+   char *name, *value, *buf;
+   
+   while (l != NULL)
+     {
+	name = l->name;
+	if (NULL == (value = l->value))
+	    {
+	       if (all) value = "";
+	       else break;
+	    }
+	
+	if (NULL == (copy = (Slrn_Header_Line_Type*) slrn_malloc
+		     (sizeof (Slrn_Header_Line_Type), 0, 1)))
+	  continue;
+	
+	len = strlen (name) + strlen (value) + 2;
+	
+	if (NULL == (buf = slrn_malloc (len, 0, 1)))
+	  {
+	     slrn_free ((char *) copy);
+	     continue;
+	  }
+	
+	copy->name = buf;
+	strcpy (buf, name); /* safe */
+	buf += strlen (name) + 1;
+	
+	copy->value = buf; 
+	strcpy (buf, value); /* safe */
+	
+	copy->name_len = l->name_len;
+	copy->next = NULL;
+	
+	if (NULL != old)
+	  old->next = copy;
+	else
+	  retval = copy;
+	
+	old = copy;
+	l = l->next;
+     }
+   
+   return retval;
+}
+
+/*}}}*/
+
 #ifndef SLRNPULL_CODE
+
 static int XOver_Done;
 static int XOver_Min;
 static int XOver_Max;
@@ -579,6 +743,7 @@ int slrn_open_xover (int min, int max) /*{{{*/
 
    if (Slrn_Server_Obj->sv_has_xover && !Suspend_XOver_For_Kill)
      {
+	rearrange_add_headers ();
 	status = Slrn_Server_Obj->sv_nntp_xover (min, max);
 	if (status == OK_XOVER)
 	  {
@@ -631,7 +796,12 @@ int slrn_open_xover (int min, int max) /*{{{*/
 static int parse_xover_line (char *buf, Slrn_XOver_Type *xov) /*{{{*/
 {
    char *b;
-   int id;
+   int id, i;
+   Slrn_Header_Line_Type *addh;
+   Overview_Fmt_Type *t = Overview_Fmt;
+   
+   for (addh = Additional_Headers; addh != NULL; addh = addh->next)
+     addh->value = NULL;
    
    b = buf;
    
@@ -639,50 +809,55 @@ static int parse_xover_line (char *buf, Slrn_XOver_Type *xov) /*{{{*/
    if (*b) *b++ = 0;
    id = atoi (buf);
    
-   Xover_Headers [SUBJECT_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-
-   Xover_Headers [FROM_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-   
-   Xover_Headers [DATE_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-   
-   Xover_Headers [MSGID_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-   
-   Xover_Headers [REFS_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-
-   Xover_Headers [BYTES_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-
-   Xover_Headers [LINES_OFFSET].value = b;
-   while (*b && (*b != '\t')) b++;
-   if (*b) *b++ = 0;
-
-   Xover_Headers [XREF_OFFSET].value = NULL;
-
-   /* Optional fields consist of Header: stuff.  Look for Xref: ... */
-   while (*b != 0)
+   for (i = 0; i < 7; ++i)
      {
-	char *xb = b;
-	     
-	/* skip to next field. */
+	Xover_Headers[i].value = b;
 	while (*b && (*b != '\t')) b++;
 	if (*b) *b++ = 0;
-	     
-	if (0 == slrn_case_strncmp ((unsigned char *) xb, 
-				    (unsigned char *) "Xref: ", 6))
+     }
+   
+   if (t == NULL)
+     /* we don't know OVERVIEW.FMT, so just look for Xref */
+     {
+	while (*b != 0)
 	  {
-	     Xover_Headers[XREF_OFFSET].value = xb + 6;
-	     break;
+	     char *xb = b;
+	     
+	     /* skip to next field. */
+	     while (*b && (*b != '\t')) b++;
+	     if (*b) *b++ = 0;
+	     
+	     if (0 == slrn_case_strncmp ((unsigned char *) xb, 
+					 (unsigned char *) "Xref: ", 6))
+	       {
+		  Xref = xb + 6;
+		  break;
+	       }
+	  }
+     }
+   else
+     {
+	while (t != NULL)
+	  {
+	     char *xb = b;
+	     
+	     while (*b && (*b != '\t')) b++;
+	     if (*b) *b++ = 0;
+	     
+	     if (t->value != NULL)
+	       {
+		  if (t->full)
+		    {
+		       if ((NULL != (xb = strchr (xb, ':'))) && (0 != *(++xb)))
+			 *t->value = ++xb;
+		       else
+			 *t->value = "";
+		    }
+		  else
+		    *t->value = xb;
+	       }
+	     
+	     t = t->next;
 	  }
      }
    
@@ -816,11 +991,9 @@ int slrn_xover_for_msgid (char *msgid, Slrn_XOver_Type *xov) /*{{{*/
 	  slrn_error ("Server does not provide this capability.");
 	return -1;
      }
-      
+   
    if (id == 0) id = -1;
    retval = read_head_into_xover (id, xov);
-   if (NULL == xov->add_hdrs)
-     xov->add_hdrs = copy_add_headers (Additional_Headers);
    
    return retval;
 }
@@ -846,9 +1019,9 @@ int slrn_open_add_xover (int min, int max) /*{{{*/
      return 0;
    
    if (Slrn_Server_Obj->sv_has_xhdr == -1)
-     Slrn_Server_Obj->sv_has_xhdr = Slrn_Server_Obj->sv_has_cmd ("XHDR");
+     Slrn_Server_Obj->sv_has_xhdr = Slrn_Server_Obj->sv_has_cmd ("XHDR Path");
    
-   if (Slrn_Server_Obj->sv_has_xhdr)
+   if (Slrn_Server_Obj->sv_has_xhdr == 1)
      {
 	status = Slrn_Server_Obj->sv_nntp_xhdr (Current_Add_Header->name,
 						min, max);
@@ -878,13 +1051,16 @@ int slrn_read_add_xover (Slrn_Header_Line_Type **l) /*{{{*/
    char *b;
    int id, status;
    
-   if (Slrn_Server_Obj->sv_has_xhdr)
+   if (Slrn_Server_Obj->sv_has_xhdr == 1)
      {
 	while (1)
 	  {
 	     status = Slrn_Server_Obj->sv_read_line (valbuf, sizeof (valbuf));
 	     if (status == -1)
-	       return -1;
+	       {
+		  Current_Add_Header = NULL;
+		  return -1;
+	       }
 	     if (status == 0)
 	       {
 		  Current_Add_Header = Current_Add_Header->next;
@@ -903,7 +1079,8 @@ int slrn_read_add_xover (Slrn_Header_Line_Type **l) /*{{{*/
 	
 	hlbuf.name = Current_Add_Header->name;
 	hlbuf.name_len = Current_Add_Header->name_len;
-	if (strcmp (b, "(none)")) /* inn returns this */
+	if (strcmp (b, "(none)") &&
+	    strcmp (b, "(null)")) /* some servers return this */
 	  hlbuf.value = b;
 	else
 	  hlbuf.value = NULL;
@@ -918,6 +1095,7 @@ int slrn_read_add_xover (Slrn_Header_Line_Type **l) /*{{{*/
    if (status == 1)
      {
 	slrn_free (xov.subject_malloced);
+	slrn_free (xov.from_malloced);
 	slrn_free_additional_headers (xov.add_hdrs);
 	return xov.id;
      }
@@ -941,53 +1119,56 @@ int slrn_add_xover_missing (void)
    return (Current_Add_Header != NULL);
 }
 
-static Slrn_Header_Line_Type *copy_add_headers (Slrn_Header_Line_Type *l) /*{{{*/
+/* makes sure that headers found in the XOVER output are listed first */
+static void rearrange_add_headers (void) /*{{{*/
 {
-   Slrn_Header_Line_Type *retval = NULL, *old = NULL, *copy;
-   unsigned int len;
-   char *name, *value, *buf;
+   Slrn_Header_Line_Type *nonnov = NULL, *last = NULL, *l = Additional_Headers;
+   Overview_Fmt_Type *o = Overview_Fmt;
    
-   while (NULL != l)
+   while (l != NULL)
      {
-	if (NULL == (copy = (Slrn_Header_Line_Type*) slrn_malloc
-		     (sizeof (Slrn_Header_Line_Type), 0, 1)))
-	  return retval;
-	
-	name = l->name;
-	if (NULL == (value = l->value))
-	  value = "";
-	
-	len = strlen (name) + strlen (value) + 2;
-	
-	if (NULL == (buf = slrn_malloc (len, 0, 1)))
-	  {
-	     slrn_free ((char *) copy);
-	     return retval;
-	  }
-	
-	copy->name = buf;
-	strcpy (buf, name); /* safe */
-	buf += strlen (name) + 1;
-	
-	copy->value = buf; 
-	strcpy (buf, value); /* safe */
-	
-	copy->name_len = l->name_len;
-	copy->next = NULL;
-
-	if (NULL != old)
-	  old->next = copy;
-	else
-	  retval = copy;
-	
-	old = copy;
+	l->value = NULL;
 	l = l->next;
      }
    
-   return retval;
+   while (o != NULL)
+     {
+	if (o->value != NULL)
+	  *o->value = "";
+	o = o->next;
+     }
+   
+   l = Additional_Headers;
+   Additional_Headers = NULL;
+   
+   while (l != NULL)
+     {
+	Slrn_Header_Line_Type *next = l->next;
+	if (l->value == NULL)
+	  {
+	     l->next = nonnov;
+	     nonnov = l;
+	  }
+	else
+	  {
+	     l->next = Additional_Headers;
+	     Additional_Headers = l;
+	     if (last == NULL)
+	       last = l;
+	  }
+	l = next;
+     }
+   
+   if (last != NULL)
+     last->next = nonnov;
+   else
+     Additional_Headers = nonnov;
+   
+   Unretrieved_Headers = Current_Add_Header = nonnov;
 }
 
 /*}}}*/
+
 void slrn_append_add_xover_to_header (Slrn_Header_Type *h, /*{{{*/
 				      Slrn_Header_Line_Type *l)
 {
@@ -997,10 +1178,10 @@ void slrn_append_add_xover_to_header (Slrn_Header_Type *h, /*{{{*/
      {
 	while (pos->next != NULL)
 	  pos = pos->next;
-	pos->next = copy_add_headers (l);
+	pos->next = copy_add_headers (l, 1);
      }
    else
-     h->add_hdrs = copy_add_headers (l);
+     h->add_hdrs = copy_add_headers (l, 1);
 }
 
 /*}}}*/
